/*******************************************************************************
 *
 * File Name: LDreferenceClass.cpp
 * Author: Richard Baxter
 * Project: Generic Ldraw Construct Functions
 * Project Version: 3e2a 16-Feb-07
 *
 *******************************************************************************/



/*LDreferenceClass.cpp: Defines a class for references to submodels/parts/units within a scene file*/

#include "LDreferenceClass.h"



Reference::Reference(void)
{
	//Define default values for Reference

	relativePosition.x = 0.0;
	relativePosition.y = 0.0;
	relativePosition.z = 0.0;
	absolutePosition.x = 0.0;
	absolutePosition.y = 0.0;
	absolutePosition.z = 0.0;
		//deformationMatrix initialisations here are required for bcc32 compiler
	deformationMatrix.a.x = 1.0;
	deformationMatrix.a.y = 0.0;
	deformationMatrix.a.z = 0.0;
	deformationMatrix.b.x = 0.0;
	deformationMatrix.b.y = 1.0;
	deformationMatrix.b.z = 0.0;
	deformationMatrix.c.x = 0.0;
	deformationMatrix.c.y = 0.0;
	deformationMatrix.c.z = 1.0;
	colour = 0;
	name = "";
	next = NULL;
#ifdef USE_LRRC
	isSubModelReference = false;
#endif

		/*new parameters added to parseFile() 18-mar-07*/
	vertex1relativePosition.x = 0.0;
	vertex1relativePosition.y = 0.0;
	vertex1relativePosition.z = 0.0;
	vertex2relativePosition.x = 0.0;
	vertex2relativePosition.y = 0.0;
	vertex2relativePosition.z = 0.0;
	vertex3relativePosition.x = 0.0;
	vertex3relativePosition.y = 0.0;
	vertex3relativePosition.z = 0.0;
	vertex4relativePosition.x = 0.0;
	vertex4relativePosition.y = 0.0;
	vertex4relativePosition.z = 0.0;
	vertex1absolutePosition.x = 0.0;
	vertex1absolutePosition.y = 0.0;
	vertex1absolutePosition.z = 0.0;
	vertex2absolutePosition.x = 0.0;
	vertex2absolutePosition.y = 0.0;
	vertex2absolutePosition.z = 0.0;
	vertex3absolutePosition.x = 0.0;
	vertex3absolutePosition.y = 0.0;
	vertex3absolutePosition.z = 0.0;
	vertex4absolutePosition.x = 0.0;
	vertex4absolutePosition.y = 0.0;
	vertex4absolutePosition.z = 0.0;
	type = REFERENCE_TYPE_UNDEFINED;

	/*Additional values used with recursive parser*/
#ifdef USE_LRRC
	subModelDetails = NULL;
	firstReferenceWithinSubModel = NULL;
#endif

	/*cout << "\nA Reference object was Constructed\n" << endl;*/
}

Reference::~Reference()
{
	//delete subModelDetails;		//this destroys memory - do not use this
    //insert explict reference to delete any memory structures allocated automatically during construction (there are currently none);
}



Reference::Reference(string referenceName, int referenceColour, bool createNewSubmodel)
{
	//Define default values for Reference

	relativePosition.x = 0.0;
	relativePosition.y = 0.0;
	relativePosition.z = 0.0;
	absolutePosition.x = 0.0;
	absolutePosition.y = 0.0;
	absolutePosition.z = 0.0;
		//deformationMatrix initialisations here are required for bcc32 compiler
	deformationMatrix.a.x = 1.0;
	deformationMatrix.a.y = 0.0;
	deformationMatrix.a.z = 0.0;
	deformationMatrix.b.x = 0.0;
	deformationMatrix.b.y = 1.0;
	deformationMatrix.b.z = 0.0;
	deformationMatrix.c.x = 0.0;
	deformationMatrix.c.y = 0.0;
	deformationMatrix.c.z = 1.0;
	colour = referenceColour;
	name = referenceName;
	next = NULL;
#ifdef USE_LRRC
	isSubModelReference = false;
#endif

		/*new parameters added to parseFile() 18-mar-07*/
	vertex1relativePosition.x = 0.0;
	vertex1relativePosition.y = 0.0;
	vertex1relativePosition.z = 0.0;
	vertex2relativePosition.x = 0.0;
	vertex2relativePosition.y = 0.0;
	vertex2relativePosition.z = 0.0;
	vertex3relativePosition.x = 0.0;
	vertex3relativePosition.y = 0.0;
	vertex3relativePosition.z = 0.0;
	vertex4relativePosition.x = 0.0;
	vertex4relativePosition.y = 0.0;
	vertex4relativePosition.z = 0.0;
	vertex1absolutePosition.x = 0.0;
	vertex1absolutePosition.y = 0.0;
	vertex1absolutePosition.z = 0.0;
	vertex2absolutePosition.x = 0.0;
	vertex2absolutePosition.y = 0.0;
	vertex2absolutePosition.z = 0.0;
	vertex3absolutePosition.x = 0.0;
	vertex3absolutePosition.y = 0.0;
	vertex3absolutePosition.z = 0.0;
	vertex4absolutePosition.x = 0.0;
	vertex4absolutePosition.y = 0.0;
	vertex4absolutePosition.z = 0.0;
	type = REFERENCE_TYPE_UNDEFINED;

	/*Additional values used with recursive parser*/
#ifdef USE_LRRC

	if(createNewSubmodel)
	{
		subModelDetails = new ModelDetails();
		firstReferenceWithinSubModel = NULL;
	}
	else
	{
		subModelDetails = NULL;
		firstReferenceWithinSubModel = NULL;
	}


#endif

	/*cout << "\nA Reference object was Constructed\n" << endl;*/
}



Reference::Reference(bool createNewSubmodel)
{
	//Define default values for Reference

	relativePosition.x = 0.0;
	relativePosition.y = 0.0;
	relativePosition.z = 0.0;
	absolutePosition.x = 0.0;
	absolutePosition.y = 0.0;
	absolutePosition.z = 0.0;
		//deformationMatrix initialisations here are required for bcc32 compiler
	deformationMatrix.a.x = 1.0;
	deformationMatrix.a.y = 0.0;
	deformationMatrix.a.z = 0.0;
	deformationMatrix.b.x = 0.0;
	deformationMatrix.b.y = 1.0;
	deformationMatrix.b.z = 0.0;
	deformationMatrix.c.x = 0.0;
	deformationMatrix.c.y = 0.0;
	deformationMatrix.c.z = 1.0;
	colour = 0;
	name = "";
	next = NULL;
#ifdef USE_LRRC
	isSubModelReference = false;
#endif
		/*new parameters added to parseFile() 18-mar-07*/
	vertex1relativePosition.x = 0.0;
	vertex1relativePosition.y = 0.0;
	vertex1relativePosition.z = 0.0;
	vertex2relativePosition.x = 0.0;
	vertex2relativePosition.y = 0.0;
	vertex2relativePosition.z = 0.0;
	vertex3relativePosition.x = 0.0;
	vertex3relativePosition.y = 0.0;
	vertex3relativePosition.z = 0.0;
	vertex4relativePosition.x = 0.0;
	vertex4relativePosition.y = 0.0;
	vertex4relativePosition.z = 0.0;
	vertex1absolutePosition.x = 0.0;
	vertex1absolutePosition.y = 0.0;
	vertex1absolutePosition.z = 0.0;
	vertex2absolutePosition.x = 0.0;
	vertex2absolutePosition.y = 0.0;
	vertex2absolutePosition.z = 0.0;
	vertex3absolutePosition.x = 0.0;
	vertex3absolutePosition.y = 0.0;
	vertex3absolutePosition.z = 0.0;
	vertex4absolutePosition.x = 0.0;
	vertex4absolutePosition.y = 0.0;
	vertex4absolutePosition.z = 0.0;
	type = REFERENCE_TYPE_UNDEFINED;

	/*Additional values used with recursive parser*/
#ifdef USE_LRRC

	if(createNewSubmodel)
	{
		subModelDetails = new ModelDetails();
		firstReferenceWithinSubModel = NULL;
	}
	else
	{
		subModelDetails = NULL;
		firstReferenceWithinSubModel = NULL;
	}


#endif

	/*cout << "\nA Reference object was Constructed\n" << endl;*/
}

void copyReferencePosition(Reference * r, Reference * referenceToCopy)
{
	r->relativePosition.x = referenceToCopy->relativePosition.x;
	r->relativePosition.y = referenceToCopy->relativePosition.y;
	r->relativePosition.z = referenceToCopy->relativePosition.z;
	r->absolutePosition.x = referenceToCopy->absolutePosition.x;
	r->absolutePosition.y = referenceToCopy->absolutePosition.y;
	r->absolutePosition.z = referenceToCopy->absolutePosition.z;
}



int obtainReferencePlayerID(Reference * referenceInThisPhaseStartSceneFile)
{
	#ifdef GAME_USE_FILENAME_AS_UNIQUE_UNIT_IDS
	return referenceInThisPhaseStartSceneFile->colour;
	#endif
	#ifdef GAME_USE_COLOUR_AS_UNIQUE_UNIT_IDS
	return (referenceInThisPhaseStartSceneFile->colour)%10;		//Eg the player ID is the least significant bits
	#endif
}
