/*******************************************************************************
 *
 * File Name: LRRCgame.cpp
 * Author: Richard Baxter
 * Project: Lego Rules CG Rounds Checker
 * Project Version: 3e2a 16-Feb-07
 * Project First Internal Release: 1aXx 18-Sept-05 (C)
 * Project Second Internal Release: 2aXx 02-April-06 (convert to C++)
 * Project Third Internal Release: 2b7d 26-Sept-06 (added sprites)
 * Project Fourth Internal Release: 2c4a 25-Feb-07 [rotation of models supported, copyCombatRelevantInheritedImmediateParentUnitDetailsIntoChildObject() no longer used]
 * Project Fifth Internal Release: 2d11a 3-June-07
 *
 * NB phase specific sprites are yet to be added to scene files based upon movement/combat outcomes
 * NB phase specific sprites are yet to be added to scene files based upon future movement/combat options
 *******************************************************************************/

#include "LDglobalDefs.h"

#include "LRRCgame.h"
#include "LRRCmovement.h"
#include "LRRCcombat.h"
#include "LRRCsprite.h"
#include "LDparser.h"
#include "LDreferenceManipulation.h"
#include "LRRCgameReferenceManipulation.h"
//#include "LDreferenceClass.h"
#include "XMLrulesClass.h"

#ifdef USE_ANN
	#include "ANNneuronClass.h"
	#include "ANNFormation.h"
	#include "LRRCgameAI.h"
	#include "LRRCunitClass.h"
	#include "ANNTraining.h"
	#include "ANNXMLconversion.h"
	#include "ANNsprite.h"
	#include "ANNUpdateAlgorithm.h"
#endif

#include <string>
#include <iostream>
#include <fstream>
#include <time.h>
#include <math.h>
using namespace std;



static int currentRound = GAME_ROUND_DEFAULT;							//1, 2, 3 .. etc
static int currentPlayerTurn = GAME_PLAYER_TURN_DEFAULT;				//1, 2, 3... etc [total number of players]
static int currentPhase = GAME_PHASE_DEFAULT;							//movement, longdist combat, or closecombat
static int currentNumberPlayers = GAME_PLAYERS_NUM_DEFAULT;		//arbitrary


static string SCENE_FILE_NAME_START;
static string SCENE_FILE_NAME_GAME_HEADER;
static string SCENE_FILE_NAME_ROUND_HEADER;
static string SCENE_FILE_NAME_PLAYER_HEADER;
static string SCENE_FILE_NAME_PHASE_START;
static string SCENE_FILE_NAME_PHASE_RANGESPRITES;
static string SCENE_FILE_NAME_PHASE_TARGETSPRITES;
static string SCENE_FILE_NAME_PHASE_END;
static string SCENE_FILE_NAME_PHASE_MOVEMENT;
static string SCENE_FILE_NAME_PHASE_LONGDISTANCE;
static string SCENE_FILE_NAME_PHASE_CLOSECOMBAT;
static string SCENE_FILE_NAME_EXTENSION;

void fillInGameExternVariables()
{
	RulesClass * currentReferenceRulesClass = LRRCrulesMiscellaneous;

	while(currentReferenceRulesClass->next != NULL)
	{
		if(currentReferenceRulesClass->name == SCENE_FILE_NAME_START_NAME)
		{
			SCENE_FILE_NAME_START = currentReferenceRulesClass->stringValue;
		}
		else if(currentReferenceRulesClass->name == SCENE_FILE_NAME_GAME_HEADER_NAME)
		{
			SCENE_FILE_NAME_GAME_HEADER = currentReferenceRulesClass->stringValue;
		}
		else if(currentReferenceRulesClass->name == SCENE_FILE_NAME_ROUND_HEADER_NAME)
		{
			SCENE_FILE_NAME_ROUND_HEADER = currentReferenceRulesClass->stringValue;
		}
		else if(currentReferenceRulesClass->name == SCENE_FILE_NAME_PLAYER_HEADER_NAME)
		{
			SCENE_FILE_NAME_PLAYER_HEADER = currentReferenceRulesClass->stringValue;
		}
		else if(currentReferenceRulesClass->name == SCENE_FILE_NAME_PHASE_START_NAME)
		{
			SCENE_FILE_NAME_PHASE_START = currentReferenceRulesClass->stringValue;
		}
		else if(currentReferenceRulesClass->name == SCENE_FILE_NAME_PHASE_RANGESPRITES_NAME)
		{
			SCENE_FILE_NAME_PHASE_RANGESPRITES = currentReferenceRulesClass->stringValue;
		}
		else if(currentReferenceRulesClass->name == SCENE_FILE_NAME_PHASE_TARGETSPRITES_NAME)
		{
			SCENE_FILE_NAME_PHASE_TARGETSPRITES = currentReferenceRulesClass->stringValue;
		}
		else if(currentReferenceRulesClass->name == SCENE_FILE_NAME_PHASE_END_NAME)
		{
			SCENE_FILE_NAME_PHASE_END = currentReferenceRulesClass->stringValue;
		}
		else if(currentReferenceRulesClass->name == SCENE_FILE_NAME_PHASE_MOVEMENT_NAME)
		{
			SCENE_FILE_NAME_PHASE_MOVEMENT = currentReferenceRulesClass->stringValue;
		}
		else if(currentReferenceRulesClass->name == SCENE_FILE_NAME_PHASE_LONGDISTANCE_NAME)
		{
			SCENE_FILE_NAME_PHASE_LONGDISTANCE = currentReferenceRulesClass->stringValue;
		}
		else if(currentReferenceRulesClass->name == SCENE_FILE_NAME_PHASE_CLOSECOMBAT_NAME)
		{
			SCENE_FILE_NAME_PHASE_CLOSECOMBAT = currentReferenceRulesClass->stringValue;
		}
		else if(currentReferenceRulesClass->name == SCENE_FILE_NAME_EXTENSION_NAME)
		{
			SCENE_FILE_NAME_EXTENSION = currentReferenceRulesClass->stringValue;
		}
		else
		{

		}

		currentReferenceRulesClass = currentReferenceRulesClass->next;
	}
}

bool executeLLRCfunctionsWithAI()
{
	bool result = true;

	int initialRound;
	int initialPlayerTurn;
	int initialPhase;
	int numberPlayers;

	if(!parseLRRCRulesXMLFile())
	{
		result = false;
	}
	fillInCombatExternVariables();
	fillInGameExternVariables();
	fillInModelClassExternVariables();
	fillInParserExternVariables();
	fillInPlayerClassExternVariables();
	fillInSpriteExternVariables();




	initialRound = GAME_ROUND_DEFAULT;
	initialPlayerTurn = GAME_PLAYER_TURN_DEFAULT;
	initialPhase = GAME_PHASE_DEFAULT;
	gameObtainNumberPlayers(&numberPlayers);

	UnitListClass * firstUnitInUnitList = new UnitListClass();
	Player * initialPlayerInList = new Player();
	generatePlayerList(numberPlayers, initialPlayerInList, firstUnitInUnitList);


#ifdef USE_ANN
	//bool AILearningEnabled = true;
	//if(AILearningEnabled)
	//{
		//fill UnitLists for each player
		char preMovementPhaseSceneFileName[SCENE_FILE_NAME_MAX_LEN];
		generateSceneFileName(GAME_NUM_DEFAULT, currentRound, currentPlayerTurn, GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_START, preMovementPhaseSceneFileName);
		parseSceneFileAndFillUnitLists(preMovementPhaseSceneFileName, firstUnitInUnitList, initialRound);


		//create AI neural network and share neural network between players (ie give each player the same neural network)

		Reference * unitReference = new Reference();
		Reference * unitRefenceOpponent = new Reference();
		int unitCombatType = COMBAT_EXPERIENCE_INPUT_TYPE_CLOSECOMBAT;

		#define IRRELEVANT (0)
		long unitDecision = IRRELEVANT;
		Experience * tempExperience = new Experience;
		generateExperienceFromUnitCombatOutcome(unitReference, unitRefenceOpponent, unitCombatType, unitDecision, tempExperience);
		long numberOfInputNeurons = countNumberOfExperienceInputs(tempExperience);
		long numberOfOutputNeurons = COMBAT_EXPERIENCE_NUMBER_OF_DIFFERENT_OUTPUT_DECISIONS;
		NeuronContainer * firstInputNeuronInNetwork = new NeuronContainer();
		NeuronContainer * firstOutputNeuronInNetwork;

		firstOutputNeuronInNetwork = formNeuralNetWithOptimisedProperties(firstInputNeuronInNetwork, numberOfInputNeurons, numberOfOutputNeurons);

		Player * currentPlayerInList = initialPlayerInList;
		while(currentPlayerInList->next != NULL)
		{
			currentPlayerInList->firstInputNeuronInNetwork = firstInputNeuronInNetwork;
			currentPlayerInList->firstOutputNeuronInNetwork = firstOutputNeuronInNetwork;
			currentPlayerInList->numberOfInputNeurons = numberOfInputNeurons;
			currentPlayerInList->numberOfOutputNeurons = numberOfOutputNeurons;

			currentPlayerInList = currentPlayerInList->next;
		}

	//}
#endif


	for(int gameNumber=0; gameNumber<GAME_NUM_MAX_NUM; gameNumber++)
	{
		if(!gamePlay(initialRound, initialPlayerTurn, initialPhase, numberPlayers, initialPlayerInList, gameNumber))
		{
			result = false;
		}
	}

}

bool executeLLRCfunctionsInOrder()
{
	bool result;

	bool UIstatus = true;
	char answerAsString[100];
	int answerAsInt;

	int initialGame;
	int initialRound;
	int initialPlayerTurn;
	int initialPhase;
	int numberPlayers;

	if(!parseLRRCRulesXMLFile())
	{
		UIstatus = false;
	}
	fillInCombatExternVariables();
	fillInGameExternVariables();
	fillInModelClassExternVariables();
	fillInParserExternVariables();
	fillInPlayerClassExternVariables();
	fillInSpriteExternVariables();


	while(UIstatus == true)
	{
		cout << "\n ---\n";
		cout << "Do you wish to \n";
		cout << " ---\n";
		cout << "1. start LRRC at an arbitrary round, player turn, and phase\n";
		cout << "2. start LRRC at round 1, player turn 1, phase 1\n";
		cout << "3. start LRRC at the current recorded round, player turn, and phase\n";
		cout << " ---\n";
		cout << "0. Exit\n";
		cout << " ---\n";
		cout <<	"Enter Answer (0,1,2,3... etc):\n\n>> ";


		cin >> answerAsString;
		answerAsInt = int(atof(answerAsString));

		if(answerAsInt == 1)
		{
			initialGame = GAME_NUM_DEFAULT;
			gameObtainRoundPlayerTurnAndPhase(&initialRound, &initialPlayerTurn, &initialPhase);
			gameObtainNumberPlayers(&numberPlayers);

			Player * initialPlayerInList = new Player();
			UnitListClass * firstUnitInUnitList = new UnitListClass();
			generatePlayerList(numberPlayers, initialPlayerInList, firstUnitInUnitList);

			if(!gamePlay(initialRound, initialPlayerTurn, initialPhase, numberPlayers, initialPlayerInList, initialGame))
			{
				result = false;
			}
			UIstatus = false;
		}
		else if(answerAsInt == 2)
		{
			initialGame = GAME_NUM_DEFAULT;
			initialRound = GAME_ROUND_DEFAULT;
			initialPlayerTurn = GAME_PLAYER_TURN_DEFAULT;
			initialPhase = GAME_PHASE_DEFAULT;
			gameObtainNumberPlayers(&numberPlayers);

			Player * initialPlayerInList = new Player();
			UnitListClass * firstUnitInUnitList = new UnitListClass();
			generatePlayerList(numberPlayers, initialPlayerInList, firstUnitInUnitList);

			if(!gamePlay(initialRound, initialPlayerTurn, initialPhase, numberPlayers, initialPlayerInList, initialGame))
			{
				result = false;
			}
			UIstatus = false;
		}
		else if(answerAsInt == 3)
		{
			initialGame = GAME_NUM_DEFAULT;
			initialRound = currentRound;
			initialPlayerTurn = currentPlayerTurn;
			initialPhase = currentPhase;
			numberPlayers = currentNumberPlayers;

			Player * initialPlayerInList = new Player();
			UnitListClass * firstUnitInUnitList = new UnitListClass();
			generatePlayerList(numberPlayers, initialPlayerInList, firstUnitInUnitList);

			if(!gamePlay(initialRound, initialPlayerTurn, initialPhase, numberPlayers, initialPlayerInList, initialGame))
			{
				result = false;
			}
			UIstatus = false;
		}
		else if(answerAsInt == 0)
		{
			UIstatus = false;
		}
		else
		{
			cout << "\nInvalid answer, please try another command\n" << endl;
		}
	}

	return result;
}

bool gameObtainNumberPlayers(int * numberOfPlayers)
{
	bool result = true;

	cout << "\nEnter Number of Players (1, 2, 3, ... MAX_NUMBER_PLAYERS) \n" << endl;

	if(!obtainUserInputInt(numberOfPlayers))
	{
		result = false;
	}

	cout << "\nNumber of players selected = " << *numberOfPlayers << "." << endl;

	return result;
}

bool gameObtainRoundPlayerTurnAndPhase(int * initialRound, int * initialPlayerTurn, int * initialPhase)
{
	bool result = true;

	cout << "Enter Round Number (1, 2, 3, ...) \n" << endl;
	if(!obtainUserInputInt(initialRound))
	{
		result = false;
	}
	cout << "\nRound Number selected = " << *initialRound << "." << endl;

	cout << "Enter Player Turn (1, 2, 3, ... MAX_NUMBER_PLAYERS) \n" << endl;
	if(!obtainUserInputInt(initialPlayerTurn))
	{
		result = false;
	}
	cout << "\nPlayer Turn selected = " << *initialPlayerTurn << "." << endl;

	cout << "Enter Phase (1 - movement, 2 - int distance combat, 3 - close combat) \n" << endl;
	if(!obtainUserInputInt(initialPhase))
	{
		result = false;
	}
	cout << "\nInitial Phase selected = " << *initialPhase << "." << endl;

	return result;
}

void generatePlayerList(int numberOfPlayers, Player * initialPlayerInList, UnitListClass * firstUnitInUnitList)
{
	Player * currentPlayer = initialPlayerInList;

	string answerAsString;

	for(int index = 1; index < numberOfPlayers; index++)
	{
	#ifdef USE_PLAYER_NAMES
		cout << "Player nuymber " << index << " is to be created.\n" << endl;
		cout << "Enter Player Name >> ";
		cin >> answerAsString;
	#else
		answerAsString = "Player" + index;
	#endif
		fillPlayerDetails(currentPlayer, answerAsString, index, PLAYER_BANK_ACCOUNT_INITIAL_DEFAULT);
		currentPlayer->firstUnitInUnitList = firstUnitInUnitList;

		//code to create a new reference object
		Player * nextPlayer = new Player();
		currentPlayer -> next = nextPlayer;
		currentPlayer = currentPlayer->next;
	}

}

bool gamePlay(int initialRound, int initialPlayerTurn, int initialPhase, int numberOfPlayers, Player * initialPlayerInList, int currentGame)
{
	bool result = true;

	int currentRound;
	int currentPlayerTurn;
	int currentPhase;
	char answerAsString[50];

	int numberOfPlayersStillAlive = numberOfPlayers;	//an arbitrary number >= 2

	currentRound = initialRound;
	while((currentRound < GAME_ROUNDS_MAX_NUM) && (result == true) && (numberOfPlayersStillAlive >= 2))
	{
		//initialRound = GAME_ROUND_DEFAULT;
		currentPlayerTurn = initialPlayerTurn;
		Player * currentPlayer = initialPlayerInList;

		numberOfPlayersStillAlive = 0;	//get ready to count numberOfPlayersStillAlive
		while((currentPlayerTurn < numberOfPlayers) && (result == true))
		{
			if(currentPlayer->status == ALIVE)
			{
				numberOfPlayersStillAlive++;

				initialPlayerTurn = GAME_PLAYER_TURN_DEFAULT;
				currentPhase = initialPhase;

			#ifdef USE_ANN
				//update UnitLists for each player
				char preMovementPhaseSceneFileName[SCENE_FILE_NAME_MAX_LEN];
				generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_START, preMovementPhaseSceneFileName);
				parseSceneFileAndUpdateUnitList(preMovementPhaseSceneFileName, initialPlayerInList->firstUnitInUnitList, currentRound);	//extract the unit list reference from an arbitrary player
				updatePlayerStatus(initialPlayerInList);
			#endif

				while((currentPhase <= GAME_PHASES_NUM_PER_TURN) && (result == true))
				{
					initialPhase = GAME_PHASE_DEFAULT;

					bool stillPerformingPlayerTurnPhaseTrials = true;
					while(stillPerformingPlayerTurnPhaseTrials == true)
					{
						if(!executePhase(currentGame, currentRound, currentPlayerTurn, currentPhase, initialPlayerInList, numberOfPlayers))
						{
							cout << "Player Turn Phase Failed. Do you wish to try again (y/n)? \n\n>> ";
							cin >> answerAsString;
							if(answerAsString[0] == 'n')
							{
								stillPerformingPlayerTurnPhaseTrials = false;
								result = false;
							}
						}
						else
						{
							stillPerformingPlayerTurnPhaseTrials = false;
						}
					}

					currentPhase++;
				}
			}
			currentPlayer = currentPlayer->next;
			currentPlayerTurn++;
		}
		currentRound++;
	}

#ifdef USE_ANN
	//now for the player which won the game, train the shared neural network with all of it's unit's experiences [CHECKTHIS FUTURE: bias experiences for alive units?]
	//do not train network with unit experiences from teams that lost the game [CHECKTHIS FUTURE: bias experiences for units from loser teams?]

	//find alive player
	Player * currentPlayer = initialPlayerInList;
	while((currentPlayer->next != NULL) && (currentPlayer->status == false))
	{
		currentPlayer = currentPlayer->next;
	}

	UnitListClass * currentUnit = currentPlayer->firstUnitInUnitList;
	while(currentUnit->next != NULL)
	{
		feedNeuralNetworkWithASetOfExperiences(currentPlayer->firstInputNeuronInNetwork, currentPlayer->firstOutputNeuronInNetwork, currentPlayer->numberOfInputNeurons, currentPlayer->numberOfOutputNeurons, currentUnit->firstExperience);
	}

	//now output the network to xml and vector graphics to be used in the next game

	string * XMLNNSceneFileName = new string();
	generateXMLNNSceneFileName(currentGame, XMLNNSceneFileName);
	cout << "generateXMLNNSceneFileName = " << XMLNNSceneFileName << endl;
	string * vectorGraphicsNNSceneFileName = new string();
	generateVectorGraphicsNNSceneFileName(currentGame, vectorGraphicsNNSceneFileName);
	cout << "vectorGraphicsNNSceneFileName = " << vectorGraphicsNNSceneFileName << endl;

	if(!writeNetXMLFile(*XMLNNSceneFileName, currentPlayer->firstInputNeuronInNetwork))
	{
		result = false;
	}
	cout << "vector graphics file name without sprites = " << vectorGraphicsNNSceneFileName << endl;


	char * nullString;

	char * charstar = new char[(*vectorGraphicsNNSceneFileName).size()+1];
	const char * constcharstar = (*vectorGraphicsNNSceneFileName).c_str();
	strcpy(charstar, constcharstar);

	createNeuralNetworkSceneFilesWithAndWithoutSprites(charstar, nullString, currentPlayer->firstInputNeuronInNetwork);

#endif

	return result;
}


bool executePhase(int currentGame, int currentRound, int currentPlayerTurn, int currentPhase, Player * initialPlayerInList, int numberOfPlayers)
{
	bool result = true;

	if(currentPhase == GAME_PHASE_MOVEMENT)
	{
		cout << "\nRound " << currentRound << " Player " << currentPlayerTurn << " Executing Phase Movement." << endl;
		if(!executeMovement(currentGame, currentRound, currentPlayerTurn, initialPlayerInList, numberOfPlayers))
		{
			result = false;
		}
	}
	else if(currentPhase == GAME_PHASE_LONGDISTANCECOMBAT)
	{
		cout << "\nRound " << currentRound << " Player " << currentPlayerTurn << " Executing Phase Long Distance Combat." << endl;
		if(!executeLongDistanceCombat(currentGame, currentRound, currentPlayerTurn, initialPlayerInList, numberOfPlayers))
		{
			result = false;
		}
	}
	else if(currentPhase == GAME_PHASE_CLOSECOMBAT)
	{
		cout << "\nRound " << currentRound << " Player " << currentPlayerTurn << " Executing Phase Close Combat." << endl;
		if(!executeCloseCombat(currentGame, currentRound, currentPlayerTurn, initialPlayerInList, numberOfPlayers))
		{
			result = false;
		}
	}
	else
	{
		result = false;
		cout << "error: illegal phase " << endl;
	}

	return result;
}


bool executeMovement(int currentGame, int currentRound, int currentPlayerTurn, Player * initialPlayerInList, int numberOfPlayers)
{
	bool result = true;

	char preMovementPhaseSceneFileName[SCENE_FILE_NAME_MAX_LEN];
	char thisPhaseStartSceneFileName[SCENE_FILE_NAME_MAX_LEN];

	char targetSpritesSceneFileName[SCENE_FILE_NAME_MAX_LEN];

	char nextSceneFile[SCENE_FILE_NAME_MAX_LEN];
	char rangeSpritesNextSceneFile[SCENE_FILE_NAME_MAX_LEN];

	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_START, preMovementPhaseSceneFileName);
	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_END, thisPhaseStartSceneFileName);

	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_TARGETSPRITES, targetSpritesSceneFileName);

	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_LONGDISTANCECOMBAT, GAME_PHASE_EXECUTION_DISPLAY_START, nextSceneFile);
	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_LONGDISTANCECOMBAT, GAME_PHASE_EXECUTION_DISPLAY_RANGESPRITES, rangeSpritesNextSceneFile);




	Player * currentPlayer = findPlayer(initialPlayerInList, currentPlayerTurn);
	currentPlayer->currentPhase = GAME_PHASE_MOVEMENT;		//this is required for compareSceneFilesMovementPhase.

	if(currentPlayer->playerIsAI == true)
	{

		//declare initial scene references
		Reference * initialReferenceInPreMovementPhaseScene = new Reference();
		Reference * topLevelReferenceInPreMovementPhaseScene = new Reference(true);
		//parse the scene files and build Reference linked lists
		if(!parseFile(preMovementPhaseSceneFileName, initialReferenceInPreMovementPhaseScene, topLevelReferenceInPreMovementPhaseScene))
		{//file does not exist
			cout << "The file: " << preMovementPhaseSceneFileName << " does not exist in the directory" << endl;
			return false;
		}

		//not complete yet
		if(!AIsearchUnitListForPhaseActionSelectionInitialisation(currentPlayerTurn, currentPhase, NULL, initialReferenceInPreMovementPhaseScene, preMovementPhaseSceneFileName, NULL, targetSpritesSceneFileName, targetSpriteListInitialReference, &numTargetSpritesAdded, initialPlayerInList, initialPlayerInList->firstUnitInUnitList))
		{
			result = false;
		}
	}
	else
	{

		//declare initial scene references
		Reference * initialReferenceInPostMovementPhaseScene = new Reference();
		Reference * initialReferenceInPreMovementPhaseScene = new Reference();
		Reference * topLevelReferenceInPostMovementPhaseScene = new Reference(true);
		Reference * topLevelReferenceInPreMovementPhaseScene = new Reference(true);
		//parse the scene files and build Reference linked lists
		if(!parseFile(thisPhaseStartSceneFileName, initialReferenceInPostMovementPhaseScene, topLevelReferenceInPostMovementPhaseScene))
		{//file does not exist
			cout << "The file: " << thisPhaseStartSceneFileName << " does not exist in the directory" << endl;
			return false;
		}
		if(!parseFile(preMovementPhaseSceneFileName, initialReferenceInPreMovementPhaseScene, topLevelReferenceInPreMovementPhaseScene))
		{//file does not exist
			cout << "The file: " << preMovementPhaseSceneFileName << " does not exist in the directory" << endl;
			return false;
		}


		//assumes preMovementPhaseSceneFileName and thisPhaseStartSceneFileName already exist
		if(!compareScenesMovementPhase(preMovementPhaseSceneFileName, initialReferenceInPreMovementPhaseScene, initialReferenceInPostMovementPhaseScene, currentPlayer, targetSpritesSceneFileName, true))
		{
			result = false;
		}
	}

	int nextPhase = GAME_PHASE_LONGDISTANCECOMBAT;
	int nextPhasePlayerTurn = currentPlayerTurn;
	if(!prepareNextPhaseSceneFiles(nextPhase, nextPhasePlayerTurn, thisPhaseStartSceneFileName, nextSceneFile, rangeSpritesNextSceneFile))
	{
		result = false;
	}

	//....*****************************************************************************************************************************
	//... CHECKTHIS (fill in rest here)
	//adds sprites to scene file indicating unit movements
	//addSpritesToSceneFileWithPhaseResults(preMovementPhaseSceneFileName, thisPhaseStartSceneFileName, thisPhaseStartSceneFileName, SPRITES_MOVEMENT_SPECIFIC);
		//NB remove all current sprites, and these new sprites should indicate unit IDs and player movement markers

			//add sprites relevant for current
			//add sprites relevant for next phase

	return result;
}


bool executeLongDistanceCombat(int currentGame, int currentRound, int currentPlayerTurn, Player * initialPlayerInList, int numberOfPlayers)
{
	bool result = true;

	char preMovementPhaseSceneFileNameMovement[SCENE_FILE_NAME_MAX_LEN];
	char thisPhaseStartSceneFileNameMovement[SCENE_FILE_NAME_MAX_LEN];

	char preCombatPhaseSceneFileName[SCENE_FILE_NAME_MAX_LEN];
	char postCombatPhaseSceneFileName[SCENE_FILE_NAME_MAX_LEN];

	char targetSpritesSceneFileName[SCENE_FILE_NAME_MAX_LEN];

	char nextSceneFile[SCENE_FILE_NAME_MAX_LEN];
	char rangeSpritesNextSceneFile[SCENE_FILE_NAME_MAX_LEN];


	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_START, preMovementPhaseSceneFileNameMovement);
	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_END, thisPhaseStartSceneFileNameMovement);

	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_LONGDISTANCECOMBAT, GAME_PHASE_EXECUTION_DISPLAY_START, preCombatPhaseSceneFileName);
	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_LONGDISTANCECOMBAT, GAME_PHASE_EXECUTION_DISPLAY_END, postCombatPhaseSceneFileName);

	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_LONGDISTANCECOMBAT, GAME_PHASE_EXECUTION_DISPLAY_TARGETSPRITES, targetSpritesSceneFileName);

	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_CLOSECOMBAT, GAME_PHASE_EXECUTION_DISPLAY_START, nextSceneFile);
	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_CLOSECOMBAT, GAME_PHASE_EXECUTION_DISPLAY_RANGESPRITES, rangeSpritesNextSceneFile);


	//declare initial scene references
	Reference * initialReferenceInThisPhaseStartScene = new Reference();
	Reference * initialReferenceInPreMovementPhaseScene = new Reference();
	Reference * topLevelReferenceInThisPhaseStartScene = new Reference(true);
	Reference * topLevelReferenceInPreMovementPhaseScene = new Reference(true);
	//parse the scene files and build Reference linked lists
	if(!parseFile(thisPhaseStartSceneFileNameMovement, initialReferenceInThisPhaseStartScene, topLevelReferenceInThisPhaseStartScene))
	{//file does not exist
		cout << "The file: " << thisPhaseStartSceneFileNameMovement << " does not exist in the directory" << endl;
		return false;
	}
	if(!parseFile(preMovementPhaseSceneFileNameMovement, initialReferenceInPreMovementPhaseScene, topLevelReferenceInPreMovementPhaseScene))
	{//file does not exist
		cout << "The file: " << preMovementPhaseSceneFileNameMovement << " does not exist in the directory" << endl;
		return false;
	}


	int nextPhase = GAME_PHASE_CLOSECOMBAT;
	int nextPhasePlayerTurn = currentPlayerTurn;
	if(!executeGenericCombat(currentPlayerTurn, GAME_PHASE_LONGDISTANCECOMBAT,  initialReferenceInPreMovementPhaseScene, initialReferenceInThisPhaseStartScene, preCombatPhaseSceneFileName, postCombatPhaseSceneFileName, targetSpritesSceneFileName, initialPlayerInList))
	{
		result = false;
	}
	if(!prepareNextPhaseSceneFiles(nextPhase, nextPhasePlayerTurn, postCombatPhaseSceneFileName, nextSceneFile, rangeSpritesNextSceneFile))
	{
		result = false;
	}



	return result;
}

bool executeCloseCombat(int currentGame, int currentRound, int currentPlayerTurn, Player * initialPlayerInList, int numberOfPlayers)
{
	bool result = true;

	char preMovementPhaseSceneFileNameMovement[SCENE_FILE_NAME_MAX_LEN];
	char thisPhaseStartSceneFileNameMovement[SCENE_FILE_NAME_MAX_LEN];

	char preCombatPhaseSceneFileName[SCENE_FILE_NAME_MAX_LEN];
	char postCombatPhaseSceneFileName[SCENE_FILE_NAME_MAX_LEN];

	char targetSpritesSceneFileName[SCENE_FILE_NAME_MAX_LEN];

	char nextSceneFile[SCENE_FILE_NAME_MAX_LEN];
	char rangeSpritesNextSceneFile[SCENE_FILE_NAME_MAX_LEN];

	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_START, preMovementPhaseSceneFileNameMovement);
	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_END, thisPhaseStartSceneFileNameMovement);

	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_CLOSECOMBAT, GAME_PHASE_EXECUTION_DISPLAY_START, preCombatPhaseSceneFileName);
	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_CLOSECOMBAT, GAME_PHASE_EXECUTION_DISPLAY_END, postCombatPhaseSceneFileName);

	generateSceneFileName(currentGame, currentRound, currentPlayerTurn, GAME_PHASE_CLOSECOMBAT, GAME_PHASE_EXECUTION_DISPLAY_TARGETSPRITES, targetSpritesSceneFileName);


	//declare initial scene references
	Reference * initialReferenceInThisPhaseStartScene = new Reference();
	Reference * initialReferenceInPreMovementPhaseScene = new Reference();
	Reference * topLevelReferenceInThisPhaseStartScene = new Reference(true);
	Reference * topLevelReferenceInPreMovementPhaseScene = new Reference(true);
	//parse the scene files and build Reference linked lists
	if(!parseFile(thisPhaseStartSceneFileNameMovement, initialReferenceInThisPhaseStartScene, topLevelReferenceInThisPhaseStartScene))
	{//file does not exist
		cout << "The file: " << thisPhaseStartSceneFileNameMovement << " does not exist in the directory" << endl;
		return false;
	}
	if(!parseFile(preMovementPhaseSceneFileNameMovement, initialReferenceInPreMovementPhaseScene, topLevelReferenceInPreMovementPhaseScene))
	{//file does not exist
		cout << "The file: " << preMovementPhaseSceneFileNameMovement << " does not exist in the directory" << endl;
		return false;
	}


	int nextPhase = GAME_PHASE_MOVEMENT;
	int nextPhasePlayerTurn;
	if(currentPlayerTurn == numberOfPlayers)
	{
		generateSceneFileName(currentGame, (currentRound+1), currentPlayerTurn, GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_START, nextSceneFile);
		generateSceneFileName(currentGame, (currentRound+1), currentPlayerTurn, GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_RANGESPRITES, rangeSpritesNextSceneFile);
		nextPhasePlayerTurn = currentPlayerTurn;
	}
	else
	{
		generateSceneFileName(currentGame, currentRound, (currentPlayerTurn+1), GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_START, nextSceneFile);
		generateSceneFileName(currentGame, currentRound, (currentPlayerTurn+1), GAME_PHASE_MOVEMENT, GAME_PHASE_EXECUTION_DISPLAY_RANGESPRITES, rangeSpritesNextSceneFile);
		nextPhasePlayerTurn = (currentPlayerTurn+1);
	}

	if(!executeGenericCombat(currentPlayerTurn, GAME_PHASE_CLOSECOMBAT, initialReferenceInPreMovementPhaseScene, initialReferenceInThisPhaseStartScene, preCombatPhaseSceneFileName, postCombatPhaseSceneFileName, targetSpritesSceneFileName, initialPlayerInList))
	{
		result = false;
	}
	if(!prepareNextPhaseSceneFiles(nextPhase, nextPhasePlayerTurn, postCombatPhaseSceneFileName, nextSceneFile, rangeSpritesNextSceneFile))
	{
		result = false;
	}

	return result;
}


bool prepareNextPhaseSceneFiles(int nextPhase, int nextPlayerTurn, char * previousPhaseSceneFile, char * nextSceneFile, char * rangeSpritesNextSceneFile)
{
	bool result = true;

	//cout << "h2" << endl;

	//create next scene file
	copyFiles(nextSceneFile, previousPhaseSceneFile);

	//cout << "h3" << endl;

	/* METHOD2;*/
	bool addTextualSpriteInfo = true;
	bool addRangeSpriteInfo = true;
	if(!addUnitDetailsSpritesToSceneFile(previousPhaseSceneFile, rangeSpritesNextSceneFile, addTextualSpriteInfo, addRangeSpriteInfo, nextPhase, nextPlayerTurn))
	{
		result = false;
	}

	return result;
}



bool executeGenericCombat(int currentPlayerTurn, int currentPhase, Reference * initialReferenceInPreMovementPhaseScene, Reference * initialReferenceInThisPhaseStartScene, char preCombatPhaseSceneFileName[], char postCombatPhaseSceneFileName[], char targetSpritesSceneFileName[], Player * initialPlayerInList)
{
	bool result = true;

	/*
	Reference * initialReferenceInSceneFile = new Reference();
	Reference * topLevelReferenceInSceneFile = new Reference(preCombatPhaseSceneFileName, 1, true);	//The information in this object is not required or meaningful, but needs to be passed into the parseFile/parseReferenceList recursive function
	if(!parseFile(preCombatPhaseSceneFileName, initialReferenceInSceneFile, topLevelReferenceInSceneFile))
	{//file does not exist
		cout << "The file: " << preCombatPhaseSceneFileName << " does not exist in the directory" << endl;
		result = false;
	}
	*/

	//cout << "DEBUG this is correct - initialReferenceInSceneFile->name = " << initialReferenceInSceneFile->name << endl;
	//cout << "DEBUG this is correct - topLevelReferenceInSceneFile.name = " << topLevelReferenceInSceneFile.name << endl;


	//assumes only preMovementPhaseSceneFileName exists. thisPhaseStartSceneFileName will be created by PC after PC computes combat outcomes
	//....
	//... CHECKTHIS (fill in rest here)

	bool UIstatus = true;
	char answerAsString[100];
	int answerAsInt;
	char unitAttackerFileName[DAT_FILE_REF_SUBMODEL_NAME_LENGTH_MAX];
	char unitDefenderFileName[DAT_FILE_REF_SUBMODEL_NAME_LENGTH_MAX];
	int unitAttackerPlayerID;
	int unitDefenderPlayerID;

	//initialise sprite list variables
	Reference * targetSpriteListInitialReference = new Reference();
	int numTargetSpritesAdded = 0;

	Player * currentPlayer = findPlayer(initialPlayerInList, currentPlayerTurn);
	if(currentPlayer->playerIsAI == true)
	{
		//not complete yet
		if(!AIsearchUnitListForPhaseActionSelectionInitialisation(currentPlayerTurn, currentPhase, initialReferenceInPreMovementPhaseScene, initialReferenceInThisPhaseStartScene, preCombatPhaseSceneFileName, postCombatPhaseSceneFileName, targetSpritesSceneFileName, targetSpriteListInitialReference, &numTargetSpritesAdded, initialPlayerInList, initialPlayerInList->firstUnitInUnitList))
		{
			result = false;
		}
	}
	else
	{
		while(UIstatus == true)
		{
			bool unitIDLocationSuccess = true;

			if(currentPhase == GAME_PHASE_CLOSECOMBAT)
			{
				cout << "Do you wish to perform close combat with a (new) unit? (y/n) \n\n>> ";
			}
			else if(currentPhase == GAME_PHASE_LONGDISTANCECOMBAT)
			{
				cout << "Do you wish to perform long distance combat with a (new) unit? (y/n) \n\n>> ";
			}
			else
			{
				result = false;
				cout << "error: illegal phase " << endl;
			}

			cin >> answerAsString;

			if(answerAsString[0] == 'n')
			{
				UIstatus = false;
			}

			if(UIstatus == true)
			{
				obtainUnitDetailsFromUserForCombat(unitAttackerFileName, unitDefenderFileName, &unitAttackerPlayerID, &unitDefenderPlayerID, initialReferenceInThisPhaseStartScene);

			#ifdef GAME_USE_COLOUR_AS_UNIQUE_UNIT_IDS
				if(!determineUnitNamesWithColoursAndCombatReady(currentPhase, unitAttackerFileName, unitDefenderFileName, &unitAttackerPlayerID, &unitDefenderPlayerID, initialReferenceInSceneFile))
				{
					cout << "unit " << unitAttackerFileName << " of the player " << currentPlayerTurn << " could not attack unit " << unitDefenderFileName << ". Please retry with another target" << endl;
					unitIDLocationSuccess = false;
				}
			#else
				if(!determineIfUnitsExistPlusCombatReady(currentPhase, unitAttackerFileName, unitDefenderFileName, unitAttackerPlayerID, unitDefenderPlayerID, initialReferenceInThisPhaseStartScene))
				{
					cout << "unit " << unitAttackerFileName << " of the player " << currentPlayerTurn << " could not attack unit " << unitDefenderFileName << ". Please retry with another target" << endl;
					unitIDLocationSuccess = false;
				}
			#endif

				if(unitAttackerPlayerID != currentPlayerTurn)
				{
					cout << "invalid answer - only units owned by the current Player Turn Player can initiate an attack" << endl;
					unitIDLocationSuccess = false;
				}


				if(unitIDLocationSuccess == true)
				{
					performGenericCombatWithTwoCombatReadyUnitsAndAddSprites(currentPlayerTurn, currentPhase, initialReferenceInPreMovementPhaseScene, initialReferenceInThisPhaseStartScene, targetSpritesSceneFileName, unitAttackerFileName, unitDefenderFileName, unitAttackerPlayerID, unitDefenderPlayerID, targetSpriteListInitialReference, &numTargetSpritesAdded, initialPlayerInList);
				}
			}
		}
	}

	//cout << "h0" << endl;

	//write reference list to file.
	if(!writeReferencesToFile(postCombatPhaseSceneFileName, initialReferenceInThisPhaseStartScene))
	{
		result = false;
	}

	//cout << "h1" << endl;

	//add target sprites to target sprites scene file
	int targetSpriteListByteArraySize = 0;
	char * targetSpriteListByteArray = new char[DAT_FILE_REF_MAX_SIZE*DAT_FILE_MAX_NUM_OF_REFERENCES];
	convertReferencesToByteArray(targetSpriteListInitialReference, targetSpriteListByteArray, &targetSpriteListByteArraySize);
	if(!addSpriteListByteArrayToSceneFile(preCombatPhaseSceneFileName, targetSpritesSceneFileName, targetSpriteListByteArray, targetSpriteListByteArraySize, numTargetSpritesAdded))
	{
		result = false;
	}
	delete targetSpriteListByteArray;

	//cout << "h4" << endl;


	/*old; inbuilt*/
	/*
	copyFiles(nextSceneFile, previousPhaseSceneFile);

	//add range sprites to range sprites next scene file
	bool addTextualSpriteInfo = true;
	bool addRangeSpriteInfo = true;

	int rangeSpriteListByteArraySize = 0;
	char rangeSpriteListByteArray[DAT_FILE_REF_MAX_SIZE*DAT_FILE_MAX_NUM_OF_REFERENCES];
	int numRangeSpritesAdded = 0;
	vec eyeCoords;
	eyeCoords.x = 0.0;
	eyeCoords.y = 0.0;
	eyeCoords.z = 0.0;
	Reference * rangeSpriteListInitialReference = new Reference();

	searchSceneRefListAddUnitDetailsSpriteForSubmodels(initialReferenceInSceneFile, rangeSpriteListInitialReference, &eyeCoords, &numRangeSpritesAdded, rangeSpritesNextSceneFile, initialReferenceInSceneFile, addTextualSpriteInfo, addRangeSpriteInfo, nextPhase, currentPlayerTurn);

	//cout << "DEBUG: qqstart" << endl;
	convertReferencesToByteArray(rangeSpriteListInitialReference, rangeSpriteListByteArray, &rangeSpriteListByteArraySize);
	//cout << "DEBUG: qqend" << endl;

	if(!addSpriteListByteArrayToSceneFile(nextSceneFile, rangeSpritesNextSceneFile, rangeSpriteListByteArray, rangeSpriteListByteArraySize, numRangeSpritesAdded))
	{
		result = false;
	}
	*/


	return result;
}



#ifdef USE_ANN

	#define DOUBLE_COMPARISON_CORRECTION (0.00001)

bool moveUnitTowardsOpponent(Reference * unitReference, Reference * opponentReference, int currentAISubPhaseMovementOnly)
{
	bool result = true;

	vec vectorBetweenUnitAndOpponent;
	subtractVectors(&vectorBetweenUnitAndOpponent, opponentReference, unitReference);

	double initialDistanceBetweenUnits = calculateTheLongRangeAttackDistanceBetweenTwoUnits(unitReference->absolutePosition, opponentReference->absolutePosition);
	double unitMaximumMoveDistance = unitReference->subModelDetails->movementSpeed;
	double longDistanceAttackBaseRange;


	if(currentAISubPhaseMovementOnly == GAME_PHASE_MOVEMENT)
	{//move to within close combat boundary, else move as close as possible to unit

		if(initialDistanceBetweenUnits < unitMaximumMoveDistance)
		{
			//position unit within the close combat boundary of the opponent
			multipleVectorByScalar(&vectorBetweenUnitAndOpponent, (initialDistanceBetweenUnits-(CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY-DOUBLE_COMPARISON_CORRECTION))/initialDistanceBetweenUnits);
		    addVectors(unitReference->relativePosition, unitReference->relativePosition, vectorBetweenUnitAndOpponent);
		}
		else
		{
			//position unit as close as possible to the opponent
			multipleVectorByScalar(&vectorBetweenUnitAndOpponent, unitMaximumMoveDistance/initialDistanceBetweenUnits);
		    addVectors(unitReference->relativePosition, unitReference->relativePosition, vectorBetweenUnitAndOpponent);
		}
	}
	else if(currentAISubPhaseMovementOnly == GAME_PHASE_AIONLY_MOVEMENT_FOR_LONGDISTANCE)
	{//move to within long distance boundary, else move as close as possible to unit

		/*OLD EASY METHOD  - assume use primary long distance combat weapon
		double unitMaximumLongRangeFireDistance = unitReference->subModelDetails->longDistanceAttackBaseRange;
		*/

		//new hard method - extract from combat.cpp;

		double closestDistanceToOpponentAfterMovementTest;
		if(unitMaximumMoveDistance > (longRangeDistanceBetweenTheTwoUnits-unit1MinimumRangeOfFire))
		{
			closestDistanceToOpponentAfterMovementTest = unit1MinimumRangeOfFire;	//+DOUBLE_COMPARISON_CORRECTION
		}
		else
		{
			closestDistanceToOpponentAfterMovementTest = longRangeDistanceBetweenTheTwoUnits-unitMaximumMoveDistance;
		}

		double longRangeDistanceBetweenTheTwoUnits;
		double unit1MinimumRangeOfFire;
		double unit1RangeModifier;

		unit1RangeModifier = (opponentReference->absolutePosition.y - unitReference->absolutePosition.y)/LDRAW_UNITS_PER_CM*RANGE_MODIFIER_BASED_ON_INCREASED_HEIGHT;
		longRangeDistanceBetweenTheTwoUnits = calculateTheLongRangeAttackDistanceBetweenTwoUnits(unitReference, opponentReference);

		unit1MinimumRangeOfFire = CLOSE_AND_LONGDISTANCE_COMBAT_BOUNDARY;

		RulesClass * currentReferenceRulesClass = LRRCrulesUnitCombatDetailsAttackLongDistance;
		RecordClass * currentReferenceRecordClass = unitReference->subModelDetails->recordOfUnitCombatDetailsAttackLongDistance;
		bool foundOptimumLongDistanceWeapon = false;
		while((currentReferenceRecordClass->next != NULL) && (!foundOptimumLongDistanceWeapon))
		{
			if(currentReferenceRecordClass->numberOfThisPartIDInTheUnit > 0)
			{
				int currentWeaponsRange = currentReferenceRulesClass->attribute5;
				double currentWeaponMaximumRangeOfFireWithModifier = currentWeaponsRange + unit1RangeModifier;

				if(calculateIfUnitIsWithinAGivenLongRangeAttackDistance(closestDistanceToOpponentAfterMovementTest, (int)currentWeaponMaximumRangeOfFireWithModifier, (int)unit1MinimumRangeOfFire))
				{
					foundOptimumLongDistanceWeapon = true;

					if(currentReferenceRulesClass->name == CATAPULT_BUCKET_NAME)
					{
						unitReference->subModelDetails->longDistanceAttackValue = currentReferenceRulesClass->attribute4 * currentReferenceRecordClass->numberOfThisPartIDInTheUnit;
					}
					else
					{
						unitReference->subModelDetails->longDistanceAttackValue = currentReferenceRulesClass->attribute4;
					}
					unitReference->subModelDetails->longDistanceAttackBaseRange = currentReferenceRulesClass->attribute5;
					unitReference->subModelDetails->hasLongDistanceCombatWeapon = true;		//not necessary, already filled in
					unitReference = true;
				}
			}

			currentReferenceRecordClass = currentReferenceRecordClass->next;
			currentReferenceRulesClass = currentReferenceRulesClass->next;
		}

		longDistanceAttackBaseRange = unit1ReferenceInThisPhaseStartSceneFile->subModelDetails->longDistanceAttackBaseRange ;
		cout << "DEBUG: longDistanceAttackBaseRange = " << longDistanceAttackBaseRange << endl;


		if(foundOptimumLongDistanceWeapon)
		{
			//position unit within firing range of the unit's best possible long range weapon
			multipleVectorByScalar(&vectorBetweenUnitAndOpponent, (initialDistanceBetweenUnits-(longDistanceAttackBaseRange-DOUBLE_COMPARISON_CORRECTION))/initialDistanceBetweenUnits);
		    addVectors(unitReference->relativePosition, unitReference->relativePosition, vectorBetweenUnitAndOpponent);

		}
		else
		{//position unit as close as possible to the opponent

			if(unitMaximumMoveDistance > (longRangeDistanceBetweenTheTwoUnits-unit1MinimumRangeOfFire))
			{//unit does not appear to have an appropriate long range weapon, do not perform a movement during this phase, GAME_PHASE_AIONLY_MOVEMENT_FOR_LONGDISTANCE
				result = false;
			}
			else
			{//position unit as close as possible to the opponent
				multipleVectorByScalar(&vectorBetweenUnitAndOpponent, unitMaximumMoveDistance/initialDistanceBetweenUnits);
		    	addVectors(unitReference->relativePosition, unitReference->relativePosition, vectorBetweenUnitAndOpponent);
			}
		}

		//not finished yet...
	}
	else
	{
		cout << "error: moveUnitTowardsOpponent(); bad currentAISubPhaseMovementOnly" << endl;
	}

	return result;
}



//required for AI player
	//cycle through each character the player owns on the board, and determine  and
		//make sure to check that a unit selected is combat ready, if a unit is selected and performs combat make it no longer combat ready
bool AIsearchUnitListForPhaseActionSelectionInitialisation(int currentPlayerTurn, int currentPhase, Reference * initialReferenceInPreMovementPhaseScene, Reference * initialReferenceInThisPhaseStartScene, char preCombatPhaseSceneFileName[], char postCombatPhaseSceneFileName[], char targetSpritesSceneFileName[], Reference * targetSpriteListInitialReference, int * numTargetSpritesAdded, Player * initialPlayerInList, UnitListClass * firstUnitInUnitGroup, UnitListClass * firstUnitInOpponentUnitGroup)
{
	bool findOpponentsForAllUnitsWithinUnitGroupToFight = true;		//assume true so far

	bool result = true;


	/*
	alternatively, can use a player's unit list to perform this search, ie;
	*/

	Player * thisPlayer = findPlayer(initialPlayerInList, currentPlayerTurn);
	UnitListClass * thisPlayerCurrentUnit = firstUnitInUnitGroup;

	while(thisPlayerCurrentUnit->next != NULL)
	{
		bool unitAllowedToPerformActionThisPhase = true;
		if(currentPhase == GAME_PHASE_LONGDISTANCECOMBAT)
		{
			if(thisPlayerCurrentUnit->unitDetails->hasPerformedAction[GAME_PHASE_MOVEMENT])
			{//rules currently do not allow a unit to both move and perform long distance attack
				unitAllowedToPerformActionThisPhase = false;
			}
		}

		if((thisPlayerCurrentUnit->team == currentPlayerTurn) && (thisPlayerCurrentUnit->status == ALIVE) && (unitAllowedToPerformActionThisPhase))
		{
			bool thisPlayerUnitIDFound = false;
			bool thisPlayerUnitFoundResult = true;

				//OLD Reference * referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent = searchReferenceListFindReference(initialReferenceInThisPhaseStartScene, thisPlayerCurrentUnit->name, thisPlayerCurrentUnit->team, &thisPlayerUnitIDFound, &thisPlayerUnitFoundResult);
				//this may not be required .... but i currently assume determiniation of unit details is required for movement a unit towards an opponent [so eg that the maximum movement distance can be determined]
			if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent, initialReferenceInThisPhaseStartScene, NULL, false))
			{
				cout << "error: cannot find unit, name = " << referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->name << " id=" << referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->colour << endl;
				result = false;
			}
			performFinalUnitClassCalculations(referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->subModelDetails);


			if(!thisPlayerUnitIDFound || !thisPlayerUnitFoundResult)
			{
				result = false;
				cout << "Error: AIsearchUnitListForPhaseActionSelectionInitialisation(): searchReferenceListFindReference cannot find reference" << endl;
			}


			if(currentPhase == GAME_PHASE_MOVEMENT)
			{//movement phase
				if(!AIsearchUnitListAndCalculateWorthOfOpponents(currentPlayerTurn, currentPhase, initialReferenceInThisPhaseStartScene, initialPlayerInList, firstUnitInOpponentUnitGroup, thisPlayerCurrentUnit, referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent))		//firstUnitInOpponentUnitGroup: first unit in an arbitrary player list, as there is only 1 unit list
				{
					result = false;
				}
				if(!AIsearchUnitListAndCalculateWorthOfOpponents(currentPlayerTurn, GAME_PHASE_AIONLY_MOVEMENT_FOR_LONGDISTANCE, initialReferenceInThisPhaseStartScene, initialPlayerInList, firstUnitInOpponentUnitGroup, thisPlayerCurrentUnit, referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent))		//firstUnitInOpponentUnitGroup: first unit in an arbitrary player list, as there is only 1 unit list
				{
					result = false;
				}
				if(!AIsearchUnitListAndCalculateWorthOfOpponents(currentPlayerTurn, GAME_PHASE_LONGDISTANCECOMBAT, initialReferenceInThisPhaseStartScene, initialPlayerInList, firstUnitInOpponentUnitGroup, thisPlayerCurrentUnit, referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent))		//firstUnitInOpponentUnitGroup: first unit in an arbitrary player list, as there is only 1 unit list
				{
					result = false;
				}
			}
			else
			{//combat phase
				if(!AIsearchUnitListAndCalculateWorthOfOpponents(currentPlayerTurn, currentPhase, initialReferenceInThisPhaseStartScene, initialPlayerInList, firstUnitInOpponentUnitGroup, thisPlayerCurrentUnit, referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent))		//firstUnitInOpponentUnitGroup: first unit in an arbitrary player list, as there is only 1 unit list
				{
					result = false;
				}

			}


			bool performedActionYet = false;
			bool stillPossibleOpponentsToTryFighting = true;
			while(!performedActionYet && stillPossibleOpponentsToTryFighting)
			{
				bool performActionWithThisUnitThisPhase = false;

				int currentAISubPhaseMovementOnly = currentPhase;

				if(currentPhase == GAME_PHASE_MOVEMENT)
				{//movement phase
					bool foundOpponent = false;

					double currentLowestError = SOME_LARGE_ERROR;
					UnitListClass * referenceForOpposingPlayerCurrentUnitMforCC = searchUnitListFindOpponentWithLowestError(currentPhase, firstUnitInOpponentUnitGroup, &currentLowestError, &foundOpponent);
					UnitListClass * referenceForOpposingPlayerCurrentUnitMforLD = searchUnitListFindOpponentWithLowestError(GAME_PHASE_AIONLY_MOVEMENT_FOR_LONGDISTANCE, firstUnitInOpponentUnitGroup, &currentLowestError, &foundOpponent);
					UnitListClass * referenceForOpposingPlayerCurrentUnitLD = searchUnitListFindOpponentWithLowestError(GAME_PHASE_LONGDISTANCECOMBAT, firstUnitInOpponentUnitGroup, &currentLowestError, &foundOpponent);

					if(!foundOpponent)
					{
						stillPossibleOpponentsToTryFighting = false;
						findOpponentsForAllUnitsWithinUnitGroupToFight = false;
					}

					if((referenceForOpposingPlayerCurrentUnit->unitDetails->selfLearningTempVarBackPropagationPassError[currentPhase] < referenceForOpposingPlayerCurrentUnit->unitDetails->selfLearningTempVarBackPropagationPassError[GAME_PHASE_LONGDISTANCECOMBAT]) && (referenceForOpposingPlayerCurrentUnit->unitDetails->selfLearningTempVarBackPropagationPassError[GAME_PHASE_AIONLY_MOVEMENT_FOR_LONGDISTANCE] < referenceForOpposingPlayerCurrentUnit->unitDetails->selfLearningTempVarBackPropagationPassError[GAME_PHASE_LONGDISTANCECOMBAT]))
					{
						//check that this particular unit would not prefer to fight long distance combat instead

						//check what kind of movement is preferred;
						if(referenceForOpposingPlayerCurrentUnit->unitDetails->selfLearningTempVarBackPropagationPassError[currentPhase] < referenceForOpposingPlayerCurrentUnit->unitDetails->selfLearningTempVarBackPropagationPassError[GAME_PHASE_AIONLY_MOVEMENT_FOR_LONGDISTANCE])
						{
							currentAISubPhaseMovementOnly = currentPhase;
						}
						else
						{
							currentAISubPhaseMovementOnly = GAME_PHASE_AIONLY_MOVEMENT_FOR_LONGDISTANCE;
						}
					}
					else
					{//do not always try performing an action with a unit during this phase
						stillPossibleOpponentsToTryFighting = false;
					}
				}
				else
				{//combat phase

					bool foundOpponent = false;

					double currentLowestError = SOME_LARGE_ERROR;
					UnitListClass * referenceForOpposingPlayerCurrentUnit = searchUnitListFindOpponentWithLowestError(currentPhase, firstUnitInOpponentUnitGroup, &currentLowestError, &foundOpponent);

					if(!foundOpponent)
					{
						stillPossibleOpponentsToTryFighting = false;
						findOpponentsForAllUnitsWithinUnitGroupToFight = false;
					}

					//always try performing an action with a unit during this phase
					performActionWithThisUnitThisPhase = true;
				}

				if(performActionThisPhase)
				{
					thisPlayerCurrentUnit->unitDetails->hasPerformedAction[currentPhase] = true;

					if(thisPlayerCurrentUnit->isUnitGroup)
					{
						if(!AIsearchUnitListForPhaseActionSelectionInitialisation(currentPlayerTurn, currentPhase, initialReferenceInPreMovementPhaseScene, initialReferenceInThisPhaseStartScene, preCombatPhaseSceneFileName, postCombatPhaseSceneFileName, targetSpritesSceneFileName, targetSpriteListInitialReference, numTargetSpritesAdded, initialPlayerInList, thisPlayerCurrentUnit->firstUnitInUnitGroup, referenceForOpposingPlayerCurrentUnit->firstUnitInUnitGroup))
						{
							referenceForOpposingPlayerCurrentUnit->selfLearningTempVarUnitHasBeenTried = true;
						}
						else
						{
							performedActionYet = true;
							referenceForOpposingPlayerCurrentUnit->selfLearningTempVarUnitHasBeenTried = true;	//this is redundant
						}
					}
					else
					{
						double randomNumberBetween0And1 = ((double(abs((short)rand())))/(ABS_OF_SHORT));
						if(randomNumberBetween0And1 > thisPlayer->creativity)
						{//if creative, do something different...

							if(currentPhase == GAME_PHASE_MOVEMENT)
							{//movement phase
								//CHECKTHIS; add code here!
								//move towards unit

									//this may not be required .... but i currently assume this is required for movement towards opponent
								if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(referenceForOpposingPlayerCurrentUnit, initialReferenceInThisPhaseStartScene, NULL, false))
								{
									cout << "error: cannot find unit, name = " << referenceForOpposingPlayerCurrentUnit->name << " id=" << referenceForOpposingPlayerCurrentUnit->colour << endl;
									result = false;
								}
								performFinalUnitClassCalculations(referenceForOpposingPlayerCurrentUnit->subModelDetails);

								vec backupOfUnitRelativePostion;
								vec backupOfUnitAbsolutePostion;
								copyVectors(backupOfUnitRelativePostion, referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->relativePosition);
								copyVectors(backupOfUnitAbsolutePostion, referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->absolutePosition);

								if(moveUnitTowardsOpponent(referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent, referenceForOpposingPlayerCurrentUnit, currentAISubPhaseMovementOnly))
								{
									//assumes initialReferenceInNewScene has been successfully created
									if(!compareScenesMovementPhase(preCombatPhaseSceneFileName, initialReferenceInThisPhaseStartScene, initialReferenceInNewScene, currentPlayer, targetSpritesSceneFileName, true))
									{
									#define ALLOW_FAULTY_MOVEMENTS_TO_BE_MADE_BY_NEURAL_NETWORK
									#ifdef ALLOW_FAULTY_MOVEMENTS_TO_BE_MADE_BY_NEURAL_NETWORK
											//currently allow faulty movements to be made by neural network...
										copyVectors(referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->relativePosition, backupOfUnitRelativePostion);
										copyVectors(referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->absolutePosition, backupOfUnitAbsolutePostion);
										referenceForOpposingPlayerCurrentUnit->selfLearningTempVarUnitHasBeenTried = true;	//this is redundant

									#else
											//if not allow faulty movements to be made by neural network:
										copyVectors(referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->relativePosition, backupOfUnitRelativePostion);
										copyVectors(referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->absolutePosition, backupOfUnitAbsolutePostion);
										referenceForOpposingPlayerCurrentUnit->selfLearningTempVarUnitHasBeenTried = true;	//this is redundant
										performedActionYet = true;

									#endif

									}
									else
									{
										thisPlayerCurrentUnit->unitDetails->hasPerformedAction[currentPhase] = true;
										performedActionYet = true;
										referenceForOpposingPlayerCurrentUnit->selfLearningTempVarUnitHasBeenTried = true;	//this is redundant

										//add experience info for units
										UnitListClass * unitMoverInUnitList = thisPlayerCurrentUnit;

										int unitActionType
										unitActionType = currentPhase;
										long unitTheoreticalBestDecisionToRecord;
										unitTheoreticalBestDecisionToRecord = currentAISubPhaseMovementOnly;
										if(!addExperienceFromUnitActionOutcome(unitReference, targetReference, unitActionType, unitTheoreticalBestDecisionToRecord, initialPlayerInList->firstUnitInUnitList))		//arbitrary player, OLD=attackerPlayer
										{
											result = false;
											cout << "Error: cannot addExperienceFromUnitActionOutcome" << endl;
										}

									}
								}
								else
								{//no exact movement can be determined which can possibly be advantagous given the phase (eg phase = move in order to perform LD attack)
									referenceForOpposingPlayerCurrentUnit->selfLearningTempVarUnitHasBeenTried = true;
								}
							}
							else
							{//combat phase
								char * unitAttackerFileName = new char[referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->name.size()+1];
								const char * constunitAttackerFileName = referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->name.c_str();
								strcpy(unitAttackerFileName, constunitAttackerFileName);

								char * unitDefenderFileName = new char[referenceForOpposingPlayerCurrentUnit->name.size()+1];
								const char * constunitDefenderFileName = referenceForOpposingPlayerCurrentUnit->name.c_str();
								strcpy(unitDefenderFileName, constunitDefenderFileName);


								//char * unitAttackerFileName = referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->name.c_str();
								//char * unitDefenderFileName = referenceForOpposingPlayerCurrentUnit->name.c_str();
								int unitAttackerPlayerID = referenceForThisPlayersCurrentUnitThatIsFindingAnOpponent->colour;
								int unitDefenderPlayerID = referenceForOpposingPlayerCurrentUnit->team;

								int combatResult = performGenericCombatWithTwoCombatReadyUnitsAndAddSprites(currentPlayerTurn, currentPhase, preMovementPhaseSceneFileNameMovement, thisPhaseStartSceneFileNameMovement, preCombatPhaseSceneFileName, postCombatPhaseSceneFileName, targetSpritesSceneFileName, unitAttackerFileName, unitDefenderFileName, unitAttackerPlayerID, unitDefenderPlayerID, initialReferenceInThisPhaseStartScene, targetSpriteListInitialReference, numTargetSpritesAdded, initialPlayerInList);

								if(!combatResult)
								{
									result = false
									performedActionYet = true;	//escape out of loop - there was an IO error
									referenceForOpposingPlayerCurrentUnit->selfLearningTempVarUnitHasBeenTried = true;	//this is redundant
								}
								else
								{
									if((combatResult == ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_BOTH_CAN_STRIKE))
									{
										referenceForOpposingPlayerCurrentUnit->selfLearningTempVarUnitHasBeenTried = true;	//this is redundant
										performedActionYet = true;
									}
									else if((combatResult == ATTACK_NEITHER_UNIT_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_STRIKES_UNIT2_CAN_STRIKE))
									{
										referenceForOpposingPlayerCurrentUnit->selfLearningTempVarUnitHasBeenTried = true;
									}
									else if((combatResult == ATTACK_NEITHER_UNITS_STRIKE_BOTH_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_STRIKES_UNIT1_CAN_STRIKE))
									{
										//allow another attempt for this unit to fight during this phase ...
									}
								}
							}
						}
					}
					else
					{

						if(thisPlayerCurrentUnit->isUnitGroup)
						{
							if(!AIsearchUnitListForPhaseActionSelectionInitialisation(currentPlayerTurn, currentPhase, initialReferenceInPreMovementPhaseScene, initialReferenceInThisPhaseStartScene, preCombatPhaseSceneFileName, postCombatPhaseSceneFileName, targetSpritesSceneFileName, targetSpriteListInitialReference, numTargetSpritesAdded, initialPlayerInList, thisPlayerCurrentUnit->firstUnitInUnitGroup, initialPlayerInList->firstUnitInUnitGroup))
							{
								result = false;
							}
						}

						referenceForOpposingPlayerCurrentUnit->selfLearningTempVarUnitHasBeenTried = true;
					}
				}

			}

		}



		thisPlayerCurrentUnit = thisPlayerCurrentUnit->next;
	}

	return findOpponentsForAllUnitsWithinUnitGroupToFight;
}



//bool AIsearchUnitListAndCalculateWorthOfOpponents(int currentPlayerTurn, int currentPhase, char preMovementPhaseSceneFileNameMovement[], char thisPhaseStartSceneFileNameMovement[], char preCombatPhaseSceneFileName[], char postCombatPhaseSceneFileName[], char targetSpritesSceneFileName[], Reference * initialReferenceInSceneFile, Reference * targetSpriteListInitialReference, int * numTargetSpritesAdded, Player * initialPlayerInList, UnitListClass * firstUnitInUnitGroup, UnitListClass * playerUnitThatIsFindingAnOpponent, Reference * referenceToPlayerUnitThatIsFindingAnOpponent)
bool AIsearchUnitListAndCalculateWorthOfOpponents(int currentPlayerTurn, int currentPhaseAIOnly, Reference * initialReferenceInThisPhaseStartScene, Player * initialPlayerInList, UnitListClass * firstUnitInOpponentUnitGroup, UnitListClass * playerUnitThatIsFindingAnOpponent, Reference * referenceToPlayerUnitThatIsFindingAnOpponent)
{
	bool result = true;

	/*
	//NEW;
	self learning algorithm update;
		work out what want to do (Eg close combat);
			for every player A unit, search through unit list and calc the error for fighting a particular unit;
			perform combat with the the unit with lowest error
	*/

	UnitListClass * currentUnit = firstUnitInOpponentUnitGroup;		//arbitrary player; as there is currently one 1 unit list, and this is referenced by all players
	UnitListClass * recordOfUnitWithLeastError;

	while(currentUnit->next != NULL)
	{
		if((currentUnit->team != currentPlayerTurn) && (currentUnit->status == ALIVE))
		{
			bool currentUnitIDFound = false;
			bool currentUnitFoundResult = true;
			Reference * referenceForOpposingPlayerCurrentUnit = searchReferenceListFindReference(initialReferenceInThisPhaseStartScene, currentUnit->name, currentUnit->team, &currentUnitIDFound, &currentUnitFoundResult);
			if(!currentUnitIDFound || !currentUnitFoundResult)
			{
				result = false;
				cout << "Error: AIsearchUnitListForPhaseActionSelectionInitialisation(): searchReferenceListFindReference cannot find reference" << endl;
			}





			Player * currentPlayer = findPlayer(initialPlayerInList, currentPlayerTurn);

				//create an experience object (inputs only) given the current unit and its envirornment
			int unitActionType;
			if(currentPhaseAIOnly == GAME_PHASE_AIONLY_MOVEMENT_FOR_LONGDISTANCE)
			{
				unitActionType = GAME_PHASE_MOVEMENT;		//correction for the fact GAME_PHASE_AIONLY_MOVEMENT_FOR_LONGDISTANCE is only use by AI, and is not a real phase , ie it is not distinct from the primary movement phase
			}
			else
			{
				unitActionType = currentPhaseAIOnly;
			}

			//int combatExperienceResult = IRRELEVANT_NOT_USED;
			long unitHypotheticalDecisionToTest;
			unitHypotheticalDecisionToTest = currentPhaseAIOnly;

			Experience * experienceWithoutKnownOutput = new Experience();
			generateExperienceFromUnitCombatOutcome(referenceToPlayerUnitThatIsFindingAnOpponent, referenceForOpposingPlayerCurrentUnit, unitActionType, unitHypotheticalDecisionToTest, experienceWithoutKnownOutput);

				//fire the current experience object inputs into the network


		#ifdef METHODB

			forwardPassStep(firstInputNeuronInNetwork);

			copyNNLayer(currentUnit->selfLearningTempVarCopyOfOutputNeurons, firstOutputNeuronInNetwork);

		#elif defined METHODC
			resetInputsAndClassTargets(currentPlayer->firstInputNeuronInNetwork, currentPlayer->firstOutputNeuronInNetwork, currentPlayer->numberOfInputNeurons, currentPlayer->numberOfOutputNeurons, experienceWithoutKnownOutput);
			double backPropagationPassError;



			storeNeuralNetworkBiasAndWeights(currentPlayer->firstInputNeuronInNetwork);


			backPropagationPassError = ANNBackPropogationPass(currentPlayer->firstInputNeuronInNetwork, currentPlayer->firstOutputNeuronInNetwork);
			/*NB - need ANNBackPropogationPass to calculate the errors of the output neurons, which is done in backwardPassStep:
				forwardPassStep(firstInputNeuronInNetwork);
				tempErrorNotUsed = calculateErrorOfBackPropPass(NeuronContainer * firstOutputNeuronInNetwork);
			*/

			/*
			//NEW;
			self learning algorithm update;
				work out what want to do (Eg close combat);
					for every player A unit, search through unit list and calc the error for fighting a particular unit;
					perform combat with the the unit with lowest error
			*/
			currentUnit->selfLearningTempVarBackPropagationPassErrorCC = backPropagationPassError;
			currentUnit->selfLearningTempVarUnitHasBeenTried = false;

			//do not record any updates to neural network - the AI agent is just working out which option is the best/has the least error - ie it is playing alternatives in its mind
			restoreNeuralNetworkWithStoredBiasAndWeights(currentPlayer->firstInputNeuronInNetwork);
		#else

			forwardPassStep(firstInputNeuronInNetwork);

			//NB calculate error of output neurons
			NeuronContainer * currentNeuron = neuronBeingAccessed;
			while(currentNeuron -> nextNeuronContainer != NULL)
			{
				calculateOutputErrorOfOutputNeuron(currentNeuron);
				currentNeuron = currentNeuron->nextNeuronContainer;
			}

			backPropagationPassError = calculateErrorOfBackPropPass(firstOutputNeuronInNetwork);

			currentUnit->selfLearningTempVarBackPropagationPassError[currentPhaseAIOnly] = backPropagationPassError;

			currentUnit->selfLearningTempVarUnitHasBeenTried = false;

		#endif


		}

		if(currentUnit->isUnitGroup)
		{
			if(!AIsearchUnitListAndCalculateWorthOfOpponents(currentPlayerTurn, currentPhaseAIOnly, initialReferenceInThisPhaseStartScene, initialPlayerInList, currentUnit->firstUnitInUnitGroup, playerUnitThatIsFindingAnOpponent, referenceToPlayerUnitThatIsFindingAnOpponent))
			{
				result = false;
			}
		}

		currentUnit = currentUnit->next;
	}


	return result;
}







#endif



int performGenericCombatWithTwoCombatReadyUnitsAndAddSprites(int currentPlayerTurn, int currentPhase, Reference * initialReferenceInPreMovementPhaseScene, Reference * initialReferenceInThisPhaseStartScene, char targetSpritesSceneFileName[], char * unitAttackerFileName, char * unitDefenderFileName, int unitAttackerPlayerID, int unitDefenderPlayerID, Reference * targetSpriteListInitialReference, int * numTargetSpritesAdded, Player * initialPlayerInList)
{
	bool result = true;

	int combatResult;
	if(currentPhase == GAME_PHASE_CLOSECOMBAT)
	{
		combatResult = performCloseCombatWithConsecutiveScenes(unitAttackerFileName, unitDefenderFileName, unitAttackerPlayerID, unitDefenderPlayerID, true, true, initialReferenceInThisPhaseStartScene, initialReferenceInPreMovementPhaseScene);
			//slight deviation from current rules: NB player 2 will not have moved in previous round [therefore player 2/the defender cannot ever get a charge advantage]
	}
	else if(currentPhase == GAME_PHASE_LONGDISTANCECOMBAT)
	{
		combatResult = performLongDistanceCombatWithConsecutiveScenes(unitAttackerFileName, unitDefenderFileName, unitAttackerPlayerID, unitDefenderPlayerID, true, false, initialReferenceInThisPhaseStartScene, initialReferenceInPreMovementPhaseScene);
	}
	else
	{
		result = false;
		cout << "error: illegal phase " << endl;
	}

	if((combatResult == ATTACK_ERROR_FILE_IO) || (combatResult == ATTACK_NEITHER_UNIT_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_STRIKES_UNIT2_CAN_STRIKE))
	{
		cout << "unit " << unitAttackerFileName << " of the player " << currentPlayerTurn << " could not attack unit " << unitDefenderFileName << ". Please retry with another target" << endl;
	}
	else
	{
		bool locationResult;

		//generate reference record for sprite info generation

		Reference * topLevelReferenceInSceneFile = new Reference(true);
		Reference * unitReference = new Reference(unitAttackerFileName, unitAttackerPlayerID, true);
		Reference * targetReference = new Reference(unitDefenderFileName, unitDefenderPlayerID, true);

		locationResult = true;

		if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(unitReference, initialReferenceInThisPhaseStartScene, topLevelReferenceInSceneFile, false))
		{
			cout << "error: cannot refind unit unitAttackerFileName=" << unitAttackerFileName << " unitAttackerPlayerID=" << unitAttackerPlayerID << endl;
			result = false;
		}
		performFinalUnitClassCalculations(unitReference->subModelDetails);

		if(!searchSceneReferenceListAndDetermineTheDetailsOfAParticularUnitSubmodel(targetReference, initialReferenceInThisPhaseStartScene, topLevelReferenceInSceneFile, false))
		{
			cout << "error: cannot refind unit unitDefenderFileName=" << unitDefenderFileName << " unitDefenderPlayerID=" << unitDefenderPlayerID << endl;
			result = false;
		}
		performFinalUnitClassCalculations(targetReference->subModelDetails);


		//CHECKTHIS: new rule: only assign - has performed combat - if combat resulted in a strike (death)
		if((combatResult == ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT1_STRIKES_UNIT1_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE))
		{
			//assign hasPerformedAction to attacker
			locationResult = false;
			searchReferenceListAssignHasPerformedCombat(currentPhase, initialReferenceInThisPhaseStartScene, unitAttackerFileName, unitAttackerPlayerID, &locationResult, &result);
			if(!locationResult)
			{
				result = false;
				cout << "error: cannot assign hasPerformedAction to unit unitAttackerFileName=" << unitAttackerFileName << " unitAttackerPlayerID=" << unitAttackerPlayerID << endl;
			}
		}

		//remove dead units from reference list:
		if((combatResult == ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT1_STRIKES_UNIT1_CAN_STRIKE))
		{
			if(locationResult)
			{
				locationResult = false;
				initialReferenceInThisPhaseStartScene = searchReferenceListRemoveReferenceAndAssignUnitDead(initialReferenceInThisPhaseStartScene, unitAttackerFileName, unitAttackerPlayerID, &locationResult, &result);
				if(!locationResult)
				{
					result = false;
					cout << "error: cannot remove dead unit unitAttackerFileName=" << unitAttackerFileName << " unitAttackerPlayerID=" << unitAttackerPlayerID << endl;
				}
			}
		}

		if((combatResult == ATTACK_UNIT2_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE))
		{
			if(locationResult)
			{
				locationResult = false;
				initialReferenceInThisPhaseStartScene = searchReferenceListRemoveReferenceAndAssignUnitDead(initialReferenceInThisPhaseStartScene, unitDefenderFileName, unitDefenderPlayerID, &locationResult, &result);
				if(!locationResult)
				{
					result = false;
					cout << "error: cannot remove dead unit unitDefenderFileName=" << unitDefenderFileName << " unitDefenderPlayerID=" << unitDefenderPlayerID << endl;
				}
			}
		}

	#ifdef USE_ANN


			//add experience info for units
			bool unitIDFound;
			unitIDFound = false;
			Player * attackerPlayer = findPlayer(initialPlayerInList, unitAttackerPlayerID);
			UnitListClass * unitAttackerInUnitList = searchUnitListFindUnit(initialPlayerInList->firstUnitInUnitList, (string)unitAttackerFileName, unitAttackerPlayerID, &unitIDFound);		//arbitrary player, OLD=attackerPlayer
			if(!unitIDFound)
			{
				result = false;
				cout << "Error: unit attacker cannot be found in unit list" << endl;
			}
			unitIDFound = false;
			Player * defenderPlayer = findPlayer(initialPlayerInList, unitDefenderPlayerID);
			UnitListClass * unitDefenderInUnitList = searchUnitListFindUnit(initialPlayerInList->firstUnitInUnitList, (string)unitDefenderFileName, unitDefenderPlayerID, &unitIDFound);		//arbitrary player, OLD=defenderPlayer
			if(!unitIDFound)
			{
				result = false;
				cout << "Error: unit defender cannot be found in unit list" << endl;
			}


			int unitCombatType = currentPhase;
			int combatExperienceResult;
			long unitTheoreticalBestDecisionToRecord;
			if((combatResult == ATTACK_NEITHER_UNIT_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_STRIKES_UNIT2_CAN_STRIKE))
			{

			}
			else if((combatResult == ATTACK_BOTH_UNITS_STRIKE_BOTH_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_UNITS_STRIKE_BOTH_CAN_STRIKE))
			{

			}
			else if((combatResult == ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT1_STRIKES_UNIT1_CAN_STRIKE))
			{
				unitTheoreticalBestDecisionToRecord = COMBAT_EXPERIENCE_OUTPUT_DECISION_ATTACK_CLOSECOMBAT;
				if(!addExperienceFromUnitActionOutcome(unitReference, targetReference, unitCombatType, unitTheoreticalBestDecisionToRecord, initialPlayerInList->firstUnitInUnitList))		//arbitrary player, OLD=attackerPlayer
				{
					result = false;
					cout << "Error: cannot addExperienceFromUnitActionOutcome" << endl;
				}
			}
			else if((combatResult == ATTACK_UNIT2_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE))
			{
				unitTheoreticalBestDecisionToRecord = COMBAT_EXPERIENCE_OUTPUT_DECISION_EVADE;
				if(!addExperienceFromUnitActionOutcome(unitReference, targetReference, unitCombatType, unitTheoreticalBestDecisionToRecord, initialPlayerInList->firstUnitInUnitList))		//arbitrary player, OLD=attackerPlayer
				{
					result = false;
					cout << "Error: cannot addExperienceFromUnitActionOutcome" << endl;
				}
			}

			if((combatResult == ATTACK_NEITHER_UNIT_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_STRIKES_UNIT2_CAN_STRIKE))
			{

			}
			else if((combatResult == ATTACK_BOTH_UNITS_STRIKE_BOTH_CAN_STRIKE) || (combatResult == ATTACK_NEITHER_UNITS_STRIKE_BOTH_CAN_STRIKE))
			{

			}
			else if((combatResult == ATTACK_UNIT2_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT2_STRIKES_UNIT2_CAN_STRIKE))
			{
				unitTheoreticalBestDecisionToRecord = COMBAT_EXPERIENCE_OUTPUT_DECISION_ATTACK_CLOSECOMBAT;
				if(!addExperienceFromUnitActionOutcome(targetReference, unitReference, unitCombatType, unitTheoreticalBestDecisionToRecord, initialPlayerInList->firstUnitInUnitList))		//arbitrary player, OLD=defenderPlayer
				{
					result = false;
					cout << "Error: cannot addExperienceFromUnitActionOutcome" << endl;
				}
			}
			else if((combatResult == ATTACK_UNIT1_STRIKES_BOTH_CAN_STRIKE) || (combatResult == ATTACK_UNIT1_STRIKES_UNIT1_CAN_STRIKE))
			{
				unitTheoreticalBestDecisionToRecord = COMBAT_EXPERIENCE_OUTPUT_DECISION_EVADE;
				if(!addExperienceFromUnitActionOutcome(targetReference, unitReference, unitCombatType, unitTheoreticalBestDecisionToRecord, initialPlayerInList->firstUnitInUnitList))		//arbitrary player, OLD=defenderPlayer
				{
					result = false;
					cout << "Error: cannot addExperienceFromUnitActionOutcome" << endl;
				}
			}


	#endif



		//add target sprite info
		bool addTextualSpriteInfo = false;
		bool addRangeSpriteInfo = true;
		bool addTargetSpriteInfo = true;
		vec eyeCoords;
		eyeCoords.x = 0.0;
		eyeCoords.y = 0.0;
		eyeCoords.z = 0.0;
		if(!determineSpriteInfoAndAddSpriteToSpriteRefList(unitReference, targetReference, targetSpriteListInitialReference, &eyeCoords, numTargetSpritesAdded, targetSpritesSceneFileName, addTextualSpriteInfo, addRangeSpriteInfo, addTargetSpriteInfo, currentPhase, currentPlayerTurn))
		{
			result = false;
		}
		delete unitReference;
		delete targetReference;

	}

	return result;
}



bool generateSceneFileName(int currentGame, int currentRound, int currentPlayerTurn, int currentPhase, int phaseExecutionStage, char * sceneFileName)
{
	bool result = true;

	char currentRoundString[GAME_ROUND_STRING_MAX_LEN];
	char currentPlayerTurnString[GAME_PLAYERS_STRING_MAX_LEN];
	char currentGameString[GAME_ROUND_STRING_MAX_LEN];

	sprintf(currentRoundString, "%d", currentRound);
	sprintf(currentPlayerTurnString, "%d", currentPlayerTurn);
	sprintf(currentGameString, "%d", currentGame);

	strcpy(sceneFileName, SCENE_FILE_NAME_START.c_str());
	strcat(sceneFileName, SCENE_FILE_NAME_GAME_HEADER.c_str());
	strcat(sceneFileName, currentGameString);
	strcat(sceneFileName, SCENE_FILE_NAME_ROUND_HEADER.c_str());
	strcat(sceneFileName, currentRoundString);
	strcat(sceneFileName, SCENE_FILE_NAME_PLAYER_HEADER.c_str());
	strcat(sceneFileName, currentPlayerTurnString);
	if(currentPhase == GAME_PHASE_MOVEMENT)
	{
		strcat(sceneFileName, SCENE_FILE_NAME_PHASE_MOVEMENT.c_str());
	}
	else if(currentPhase == GAME_PHASE_LONGDISTANCECOMBAT)
	{
		strcat(sceneFileName, SCENE_FILE_NAME_PHASE_LONGDISTANCE.c_str());
	}
	else if(currentPhase == GAME_PHASE_CLOSECOMBAT)
	{
		strcat(sceneFileName, SCENE_FILE_NAME_PHASE_CLOSECOMBAT.c_str());
	}
	else
	{
		result = false;
		cout << "error: illegal phase " << endl;
	}
	if(phaseExecutionStage ==  GAME_PHASE_EXECUTION_DISPLAY_START)
	{
		strcat(sceneFileName,  SCENE_FILE_NAME_PHASE_START.c_str());
	}
	else if(phaseExecutionStage == GAME_PHASE_EXECUTION_DISPLAY_END)
	{
		strcat(sceneFileName, SCENE_FILE_NAME_PHASE_END.c_str());
	}
	else if(phaseExecutionStage == GAME_PHASE_EXECUTION_DISPLAY_TARGETSPRITES)
	{
		strcat(sceneFileName, SCENE_FILE_NAME_PHASE_TARGETSPRITES.c_str());
	}
	else if(phaseExecutionStage == GAME_PHASE_EXECUTION_DISPLAY_RANGESPRITES)
	{
		strcat(sceneFileName, SCENE_FILE_NAME_PHASE_RANGESPRITES.c_str());
	}
	else
	{
		result = false;
	}
	strcat(sceneFileName, SCENE_FILE_NAME_EXTENSION.c_str());

	return result;
}


void generateXMLNNSceneFileName(int currentGame, string * sceneFileName)
{
	char currentGameString[100];
	sprintf(currentGameString, "%d", currentGame);

	*sceneFileName = *sceneFileName + SCENE_FILE_NAME_START;
	*sceneFileName = *sceneFileName + SCENE_FILE_NAME_GAME_HEADER;
	*sceneFileName = *sceneFileName + currentGameString;
	*sceneFileName = *sceneFileName + XML_NN_SCENE_FILE_NAME_EXTENSION;
}


void generateVectorGraphicsNNSceneFileName(int currentGame, string * sceneFileName)
{
	char currentGameString[100];
	sprintf(currentGameString, "%d", currentGame);

	*sceneFileName = *sceneFileName + SCENE_FILE_NAME_START;
	*sceneFileName = *sceneFileName + SCENE_FILE_NAME_GAME_HEADER;
	*sceneFileName = *sceneFileName + currentGameString;
	*sceneFileName = *sceneFileName + VECGRAPHICS_NN_SCENE_FILE_NAME_EXTENSION;
}









/*
-----------------------------------------

Newer Rules

p1mov
	scenefileismodified
p1LDsettingAndOperation [if no movement]
	scenefileismodified
p1CCsettingAndOperation [NB Eg if p1 character x decides to fight p2 character y in close combat, p2 will fight back in CC during p1's phase [as part of standard CC operations], and p2 character y can still move in their movement phase,
	scenefileismodified
p2mov
	scenefileismodified
p2LDsettingAndOperation [if no movement]
	scenefileismodified
p2CCsettingAndOperation
	scenefileismodified
p3mov
	scenefileismodified
p3LDsettingAndOperation [if no movement]
	scenefileismodified
p3CCsettingAndOperation
	scenefileismodified

-----------------------------------------

old2 Rules [doesnt work, if P2 char y knocks P1 char x out in combat - either LD or CC, p3 char z's specified attack against p1 char x will no longer be valid];
	{assume p1 movement phase is before p2 movement phase, if p2 character y has moved longo close combat with p1 character x, p1 character x cannot fire int distance against p2 character y [even if p1 character x has not moved in his movement phase]}
	{however, if in another scenario p2 character y has moved out of range with p1 character x [after p1 character x had already specified to attack int distance], p1 character x can still fire int distance attack}

p1mov
p1LDsetting [if no movement]
p1CCsetting [NB Eg if p1 character x decides to fight p2 character y in close combat, p2 character y cannot move in their movement phase].
	scenefileismodified
p2mov
p2LDsetting [if no movement]
p2CCsetting
	scenefileismodified
p3mov
p3LDsetting [if no movement]
p3CCsetting
	scenefileismodified
...
LDoperations
	scenefileismodified
CCoperations
	scenefileismodified

-----------------------------------------

Old1 Rules:
	{assume p1 movement phase is before p2 movement phase, if p2 character y has moved longo close combat with p1 character x, p1 character x can still fire int distance against p2 character y [assuming p1 character x has not moved in his movement phase]}

p1mov
p1CCinitialsetting [NB Eg if p1 character x decides to fight p2 character y in close combat, p2 character y cannot move in their movement phase].
	scenefileismodified
p2mov
p2CCinitialsetting
	scenefileismodified
p3mov
p3CCinitialsetting
	scenefileismodified
...
p1LDsetting [if no movement]
p2LDsetting [if no movement]
p3LDsetting [if no movement]
LDoperations
	scenefileismodified
...
[NOTREQUIRED: p1CCfinalsetting]
[NOTREQUIRED: p2CCfinalsetting]
[NOTREQUIRED: p3CCfinalsetting]
CCoperations
	scenefileismodified

being first to move in a round gives the ability to lock other players longo CC.
being last to move in a round gives the ability to avoid combat.

-----------------------------------------
*/
