/*******************************************************************************
 *
 * File Name: LRRCgameAI.cpp
 * Author: Richard Baxter
 * Project: Lego Rules CG Rounds Checker
 * Project Version: 3e2a 16-Feb-07
 * Project First Internal Release: 1aXx 18-Sept-05 (C)
 * Project Second Internal Release: 2aXx 02-April-06 (convert to C++)
 * Project Third Internal Release: 2b7d 26-Sept-06 (added sprites)
 * Project Fourth Internal Release: 2c4a 25-Feb-07 [rotation of models supported, copyCombatRelevantInheritedImmediateParentUnitDetailsIntoChildObject() no longer used]
 * Project Fifth Internal Release: 2d11a 3-June-07
 *
 * NB phase specific sprites are yet to be added to scene files based upon movement/combat outcomes
 * NB phase specific sprites are yet to be added to scene files based upon future movement/combat options
 *******************************************************************************/

#include "LRRCgameAI.h"
#include "LDparser.h"
#include "LRRCgameReferenceManipulation.h"
#include "XMLrulesClass.h"
#include "LRRCcombat.h"

void parseSceneFileAndFillUnitLists(char sceneFileName[], UnitListClass * firstUnitInUnitList, int currentRound)
{
	bool result = true;

	Reference * initialReferenceInSceneFile = new Reference();
	Reference * topLevelReferenceInSceneFile = new Reference(sceneFileName, 1, true);	//The information in this object is not required or meaningful, but needs to be passed into the parseFile/parseReferenceList recursive function
	if(!parseFile(sceneFileName, initialReferenceInSceneFile, topLevelReferenceInSceneFile))
	{//file does not exist
		cout << "The file: " << sceneFileName << " does not exist in the directory" << endl;
		result = false;
	}

	fillUnitList(initialReferenceInSceneFile, firstUnitInUnitList, currentRound);


}





void fillUnitList(Reference * currentReferenceInSceneFile, UnitListClass * firstUnitInUnitGroup, int currentRound)
{
	Reference * currentReference = currentReferenceInSceneFile;
	UnitListClass * currentUnitInUnitList = firstUnitInUnitGroup;
	while(currentReference->next != NULL)
	{
		if(currentReference->isSubModelReference)
		{
			if(currentReference->subModelDetails->numPerson >= 1)
			{//add a unit to the unit list if it contains a single person

				//add a unit/group to the unit list

				currentUnitInUnitList->name = currentReference->name;
				currentUnitInUnitList->team = currentReference->colour;	//not currently used, as there is 1 UnitListClass created for every team
				currentUnitInUnitList->unitDetails = currentReference->subModelDetails;
				currentUnitInUnitList->unitDetails->status = true;
				currentUnitInUnitList->roundSpawned = currentRound;

				UnitListClass * newUnitList = new UnitListClass();
				currentUnitInUnitList->next = newUnitList;


				//go down further into reference tree
				UnitListClass * newUnitList1 = new UnitListClass();
				currentUnitInUnitList->firstUnitInUnitGroup = newUnitList1;
				currentUnitInUnitList->isUnitGroup = true;

				if(currentReference->subModelDetails->numPerson == 1)
				{
					//do not go down further into reference tree if already reached a unit containing only a single person
				}
				else
				{
					fillUnitList(currentReference->firstReferenceWithinSubModel, currentUnitInUnitList->firstUnitInUnitGroup, currentRound);
				}
			}
		}

		currentReference = currentReference->next;
		currentUnitInUnitList = currentUnitInUnitList->next;
	}
}





void parseSceneFileAndUpdateUnitList(char sceneFileName[], UnitListClass * firstUnitInUnitList, int currentRound)
{
	bool result = true;

	Reference * initialReferenceInSceneFile = new Reference();
	Reference * topLevelReferenceInSceneFile = new Reference(sceneFileName, 1, true);	//The information in this object is not required or meaningful, but needs to be passed into the parseFile/parseReferenceList recursive function
	if(!parseFile(sceneFileName, initialReferenceInSceneFile, topLevelReferenceInSceneFile))
	{//file does not exist
		cout << "The file: " << sceneFileName << " does not exist in the directory" << endl;
		result = false;
	}

	updateUnitList(initialReferenceInSceneFile, firstUnitInUnitList, currentRound);
}

void updateUnitList(Reference * initialReferenceInSceneFile, UnitListClass * firstUnitInUnitList, int currentRound)	//required after every round
{
	updateUnitListWithNewUnits(initialReferenceInSceneFile, firstUnitInUnitList, currentRound);

	updateUnitListWithDeadUnits(initialReferenceInSceneFile, firstUnitInUnitList, currentRound);

}

void updateUnitListWithNewUnits(Reference * currentReferenceInSceneFile, UnitListClass * firstUnitInUnitGroup, int currentRound)
{
	Reference * currentReference = currentReferenceInSceneFile;
	while(currentReference->next != NULL)
	{
		if(currentReference->isSubModelReference)
		{
			//add a unit to the unit list if it contains a single person

			bool foundUnitInList = false;
			UnitListClass * foundUnit;
			foundUnit = searchUnitListForUnitAndIfNotThereAddToList(firstUnitInUnitGroup, currentReference, currentRound, true, &foundUnitInList);

			//go down further into reference tree
			if(foundUnitInList)
			{
				if(foundUnit->isUnitGroup)
				{//do nothing

				}
				else
				{
					foundUnit->isUnitGroup = true;
					UnitListClass * newUnit = new UnitListClass();
					foundUnit->firstUnitInUnitGroup = newUnit;
				}
			}
			else
			{
				foundUnit->isUnitGroup = true;
				UnitListClass * newUnit = new UnitListClass();
				foundUnit->firstUnitInUnitGroup = newUnit;

			}
			updateUnitListWithNewUnits(currentReference->firstReferenceWithinSubModel, foundUnit->firstUnitInUnitGroup, currentRound);
		}

		currentReference = currentReference->next;
	}
}

void updateUnitListWithDeadUnits(Reference * initialReferenceInSceneFile, UnitListClass * firstUnitInUnitGroup, int currentRound)
{
	UnitListClass * currentUnitInList = firstUnitInUnitGroup;

	while(currentUnitInList->next != NULL)
	{
		bool unitIDFound = false;
		bool result = true;

		Reference * foundReference = searchReferenceListFindReference(initialReferenceInSceneFile, currentUnitInList->name, currentUnitInList->team, &unitIDFound, &result);
		if(!unitIDFound)
		{
			currentUnitInList->unitDetails->status = false;
			currentUnitInList->roundRemoved = currentRound;
		}

		if(currentUnitInList->isUnitGroup)
		{
			updateUnitListWithDeadUnits(initialReferenceInSceneFile, currentUnitInList->firstUnitInUnitGroup, currentRound);
		}

		currentUnitInList = currentUnitInList->next;
	}
}

UnitListClass * searchUnitListForUnitAndIfNotThereAddToList(UnitListClass * firstUnitInUnitGroup, Reference * unitReferenceInSceneFile, int currentRound, bool topLevelInTree, bool * foundUnitInList)
{
	UnitListClass * currentUnitInList = firstUnitInUnitGroup;
	UnitListClass * foundUnit;

	while((currentUnitInList->next != NULL) && (!(*foundUnitInList)))
	{
		if((currentUnitInList->name == unitReferenceInSceneFile->name) && (currentUnitInList->team == unitReferenceInSceneFile->colour))
		{
			*foundUnitInList = true;
			foundUnit = currentUnitInList;
		}
		else
		{
			if(currentUnitInList->isUnitGroup)
			{
				UnitListClass * tempUnit;
				tempUnit = searchUnitListForUnitAndIfNotThereAddToList(currentUnitInList->firstUnitInUnitGroup, unitReferenceInSceneFile, currentRound, false, foundUnitInList);
				if(*foundUnitInList)
				{
					foundUnit = tempUnit;
				}
			}
		}


		if((currentUnitInList->next == NULL) && (!(*foundUnitInList)) && (topLevelInTree))
		{
			foundUnit = currentUnitInList;

			currentUnitInList->name = unitReferenceInSceneFile->name;
			currentUnitInList->team = unitReferenceInSceneFile->colour;	//not currently used, as there is 1 UnitListClass created for every team
			currentUnitInList->unitDetails = unitReferenceInSceneFile->subModelDetails;
			currentUnitInList->unitDetails->status = true;
			currentUnitInList->roundSpawned = currentRound;

			UnitListClass * newUnitList = new UnitListClass();
			currentUnitInList->next = newUnitList;


		}

		currentUnitInList = currentUnitInList->next;

	}

	return foundUnit;
}





/*
//not used
void addExperienceToPlayer(Experience * experience, Player * player)
{
	player->currentExperience = experience;

	Experience * newExperience = new Experience();
	player->currentExperience->next = newExperience;
	player->currentExperience = player->currentExperience->next;
}

//not used
void addExperienceToUnit(Experience * experience, ModelDetails * unit)
{
	unit->currentExperience = experience;

	Experience * newExperience = new Experience();
	unit->currentExperience->next = newExperience;
	unit->currentExperience = unit->currentExperience->next;
}

//not used
void addExperienceInputToExperience(ExperienceInput * experienceInput, Experience * experience)
{
	experience->currentExperienceInput = experienceInput;

	ExperienceInput * newExperienceInput = new ExperienceInput();
	currentExperienceInput->next = newExperienceInput;
	currentExperienceInput = currentExperienceInput->next;
}
*/




//addExperienceFromUnitActionOutcome(unit, unitOpponent, unitCombatType, combatExperienceResult);




bool addExperienceFromUnitActionOutcome(Reference * unitReference, Reference * unitReferenceOpponent, int unitCombatType, long unitDecision, UnitListClass * firstUnitInUnitList)
{
	bool result = true;

	bool unitIDFound = false;
	UnitListClass * unitInUnitList = searchUnitListFindUnit(firstUnitInUnitList, unitReference->name, unitReference->colour, &unitIDFound);
	if(unitIDFound)
	{
		Experience * currentExperience = unitInUnitList->currentExperience;

		generateExperienceFromUnitCombatOutcome(unitReference, unitReferenceOpponent, unitCombatType, unitDecision, currentExperience);

		Experience * newExperience = new Experience();
		unitInUnitList->currentExperience->next = newExperience;
		unitInUnitList->currentExperience = unitInUnitList->currentExperience->next;

	}
	else
	{
		cout << "Error: unit not found in unit list" << endl;
		result = false;
	}


	return result;
}


void generateExperienceFromUnitCombatOutcome(Reference * unitReference, Reference * unitReferenceOpponent, int unitCombatType, long unitDecision, Experience * currentExperience)
{

	/*
	now store every property of the combat situation as experience inputs:

	define an experience inputs for a unit:

	unit1 recordOfUnitTypeDetails
	unit1 recordOfUnitCombatDetailsAttackCloseCombat
	unit1 recordOfUnitCombatDetailsAttackLongDistance
	unit2 recordOfUnitTypeDetails
	unit2 recordOfUnitCombatDetailsAttackCloseCombat
	unit2 recordOfUnitCombatDetailsAttackLongDistance
	unitCombatType
	combatExperienceResult

		only assume the AI Agent knows the properties of the unit.
		do not assume the AI Agent knows the rules of the game; ie do not save high level information Eg attackClass, defenceClass, etc
	*/

	currentExperience->classTargetValue = unitDecision;
	ExperienceInput * currentExperienceInput = currentExperience->firstExperienceInput;

	ModelDetails * unitDetails = unitReference->subModelDetails;
	ModelDetails * unitDetailsOpponent = unitReferenceOpponent->subModelDetails;

	RecordClass * currentRecord;

	currentRecord = unitDetails->recordOfUnitTypeDetails;
	while(currentRecord->next != NULL)
	{
		currentExperienceInput->inputValue = normaliseExperienceInput((double)currentRecord->numberOfThisPartIDInTheUnit, COMBAT_NUMBER_OF_A_PART_IN_A_UNIT_MAX);

		ExperienceInput * newExperienceInput = new ExperienceInput();
		currentExperienceInput->next = newExperienceInput;
		currentExperienceInput = currentExperienceInput->next;

		currentRecord = currentRecord->next;
	}

	currentRecord = unitDetails->recordOfUnitCombatDetailsAttackCloseCombat;
	while(currentRecord->next != NULL)
	{
		currentExperienceInput->inputValue = normaliseExperienceInput((double)currentRecord->numberOfThisPartIDInTheUnit, COMBAT_NUMBER_OF_A_PART_IN_A_UNIT_MAX);

		ExperienceInput * newExperienceInput = new ExperienceInput();
		currentExperienceInput->next = newExperienceInput;
		currentExperienceInput = currentExperienceInput->next;

		currentRecord = currentRecord->next;
	}

	currentRecord = unitDetails->recordOfUnitCombatDetailsAttackLongDistance;
	while(currentRecord->next != NULL)
	{
		currentExperienceInput->inputValue = normaliseExperienceInput((double)currentRecord->numberOfThisPartIDInTheUnit, COMBAT_NUMBER_OF_A_PART_IN_A_UNIT_MAX);

		ExperienceInput * newExperienceInput = new ExperienceInput();
		currentExperienceInput->next = newExperienceInput;
		currentExperienceInput = currentExperienceInput->next;

		currentRecord = currentRecord->next;
	}


	currentRecord = unitDetailsOpponent->recordOfUnitTypeDetails;
	while(currentRecord->next != NULL)
	{
		currentExperienceInput->inputValue = normaliseExperienceInput((double)currentRecord->numberOfThisPartIDInTheUnit, COMBAT_NUMBER_OF_A_PART_IN_A_UNIT_MAX);

		ExperienceInput * newExperienceInput = new ExperienceInput();
		currentExperienceInput->next = newExperienceInput;
		currentExperienceInput = currentExperienceInput->next;

		currentRecord = currentRecord->next;
	}

	currentRecord = unitDetailsOpponent->recordOfUnitCombatDetailsAttackCloseCombat;
	while(currentRecord->next != NULL)
	{
		currentExperienceInput->inputValue = normaliseExperienceInput((double)currentRecord->numberOfThisPartIDInTheUnit, COMBAT_NUMBER_OF_A_PART_IN_A_UNIT_MAX);

		ExperienceInput * newExperienceInput = new ExperienceInput();
		currentExperienceInput->next = newExperienceInput;
		currentExperienceInput = currentExperienceInput->next;

		currentRecord = currentRecord->next;
	}

	currentRecord = unitDetailsOpponent->recordOfUnitCombatDetailsAttackLongDistance;
	while(currentRecord->next != NULL)
	{
		currentExperienceInput->inputValue = normaliseExperienceInput((double)currentRecord->numberOfThisPartIDInTheUnit, COMBAT_NUMBER_OF_A_PART_IN_A_UNIT_MAX);

		ExperienceInput * newExperienceInput = new ExperienceInput();
		currentExperienceInput->next = newExperienceInput;
		currentExperienceInput = currentExperienceInput->next;

		currentRecord = currentRecord->next;
	}


	currentExperienceInput->inputValue = normaliseExperienceInput((double)unitCombatType, GAME_PHASES_NUM_PER_TURN);
	ExperienceInput * newExperienceInput1 = new ExperienceInput();
	currentExperienceInput->next = newExperienceInput1;
	currentExperienceInput = currentExperienceInput->next;

	/*
	currentExperienceInput->inputValue = normaliseExperienceInput((double)combatExperienceResult, COMBAT_EXPERIENCE_INPUT_RESULT_MAX);
	ExperienceInput * newExperienceInput2 = new ExperienceInput();
	currentExperienceInput->next = newExperienceInput2;
	currentExperienceInput = currentExperienceInput->next;
	*/

		int maximumMovementDistanceForAnyUnit = 0;
		RulesClass * currentReferenceRulesClass = LRRCrulesUnitTypeCatagories;
		while(currentReferenceRulesClass->next != NULL)
		{
			maximumMovementDistanceForAnyUnit = maxInt(maximumMovementDistanceForAnyUnit, currentReferenceRulesClass->attribute4);

			currentReferenceRulesClass = currentReferenceRulesClass->next;
		}
		cout << "DEBUG: maximumMovementDistanceForAnyUnit = " << maximumMovementDistanceForAnyUnit << endl;


	double distanceMovedByUnit2DuringRound = calculateTheDistanceBetweenTwoPoints(unitReference->absolutePosition, unitReferenceOpponent->absolutePosition);
	currentExperienceInput->inputValue = normaliseExperienceInput(distanceMovedByUnit2DuringRound, maximumMovementDistanceForAnyUnit);
	ExperienceInput * newExperienceInput3 = new ExperienceInput();
	currentExperienceInput->next = newExperienceInput3;
	currentExperienceInput = currentExperienceInput->next;

}


double normaliseExperienceInput(double currentInputValue, double maxInputValue)
{
	double minInputValue = 0.0;
	if(currentInputValue > maxInputValue)
	{//deal with redundant situations, eg a single unit has two swords - nb only one part type can be used per unit,  and so the following is a valid correction;
		currentInputValue = maxInputValue;
	}
	return ((currentInputValue - minInputValue) / (maxInputValue - minInputValue));
}



















void updatePlayerStatus(Player * initialPlayerInList)
{
	Player * currentPlayer = initialPlayerInList;

	while(currentPlayer->next != NULL)
	{
		if(!searchUnitListAndCheckThatSomePlayerUnitsAreAlive(currentPlayer->id, initialPlayerInList->firstUnitInUnitList))		//arbitrary player, OLD=defenderPlayer
		{
			currentPlayer->status = false;
		}

		currentPlayer = currentPlayer->next;
	}

}



bool searchUnitListAndCheckThatSomePlayerUnitsAreAlive(int playerTeam, UnitListClass * firstUnitInUnitGroup)
{
	bool playerStillHasAUnitAlive = false;

	UnitListClass * currentUnitInList = firstUnitInUnitGroup;

	while(currentUnitInList->next != NULL)
	{
		if(currentUnitInList->team = playerTeam)
		{
			playerStillHasAUnitAlive = true;
		}

		if(currentUnitInList->isUnitGroup)
		{
			if(searchUnitListAndCheckThatSomePlayerUnitsAreAlive(playerTeam, currentUnitInList->firstUnitInUnitGroup))
			{
				playerStillHasAUnitAlive = true;
			}
		}

		currentUnitInList = currentUnitInList->next;
	}

	return playerStillHasAUnitAlive;

}
