/*******************************************************************************
 *
 * File Name: LRRCunitClass.cpp
 * Author: Richard Baxter
 * Project: Lego Rules CG Rounds Checker
 * Project Version: 3e2a 16-Feb-07
 * Project First Internal Release: 1aXx 18-Sept-05 (C)
 * Project Second Internal Release: 2aXx 02-April-06 (convert to C++)
 * Project Third Internal Release: 2b7d 26-Sept-06 (added sprites)
 * Project Fourth Internal Release: 2c4a 25-Feb-07 [rotation of models supported, copyCombatRelevantInheritedImmediateParentUnitDetailsIntoChildObject() no longer used]
 * Project Fifth Internal Release: 2d11a 3-June-07
 *
 * NB phase specific sprites are yet to be added to scene files based upon movement/combat outcomes
 * NB phase specific sprites are yet to be added to scene files based upon future movement/combat options
 *******************************************************************************/

#include "LRRCunitClass.h"



UnitListClass::UnitListClass(void)
{
	roundSpawned = 0;
	roundRemoved = 0;

	name = "";
	team = 0;	//team
	//id = 0;
	unitDetails = NULL;;

	isUnitGroup = false;
	firstUnitInUnitGroup = NULL;

	firstExperience = new Experience();
	currentExperience = firstExperience;

	for(int phase = 0; phase < GAME_NUM_AI_TURN_PHASES; phase++)
	{
		selfLearningTempVarBackPropagationPassError[phase] = false;
	}
	selfLearningTempVarUnitHasBeenTried = false;

	next = NULL;
}

UnitListClass::~UnitListClass()
{
	//insert explict reference to delete any memory structures allocated automatically during construction (there are currently none);
}



UnitListClass * searchUnitListFindUnit(UnitListClass * firstUnitInUnitGroup, string unitName, int unitColour, bool * unitIDFound)
{
	UnitListClass * currentUnitInList = firstUnitInUnitGroup;
	UnitListClass * unitFound;

	while((currentUnitInList->next != NULL) && (*unitIDFound == false))
	{
		if((currentUnitInList->name == unitName) && (currentUnitInList->team == unitColour))
		{
			*unitIDFound = true;
			unitFound = currentUnitInList;
		}
		else
		{
			if(currentUnitInList->isUnitGroup)
			{
				UnitListClass * tempUnit;
				tempUnit = searchUnitListFindUnit(currentUnitInList, unitName, unitColour, unitIDFound);
				if(*unitIDFound)
				{
					unitFound = tempUnit;
				}
			}
		}

		currentUnitInList = currentUnitInList->next;
	}

	return unitFound;

}



void addUnitToList(UnitListClass * firstUnitInUnitList, string unitName, int unitColour, ModelDetails * unitDetails, int currentRound)
{
	UnitListClass * currentUnitInList = firstUnitInUnitList;

	while(currentUnitInList->next != NULL)
	{
		if(currentUnitInList->next == NULL)
		{
			currentUnitInList->name = unitName;
			currentUnitInList->team = unitColour;	//not currently used, as there is 1 UnitListClass created for every team
			currentUnitInList->unitDetails = unitDetails;


			currentUnitInList->unitDetails->status = true;
			currentUnitInList->roundSpawned = currentRound;

			UnitListClass * newUnitList = new UnitListClass();
			currentUnitInList->next = newUnitList;
		}

		currentUnitInList = currentUnitInList->next;
	}


}

UnitListClass * searchUnitListFindOpponentWithLowestError(int currentPhase, UnitListClass * firstUnitInUnitGroup, double * currentLowestError, bool * foundOpponent)
{
	UnitListClass * currentUnitInList = firstUnitInUnitGroup;
	UnitListClass * unitFound;

	while(currentUnitInList->next != NULL)
	{
		if(currentUnitInList->selfLearningTempVarUnitHasBeenTried == false)
		{
			*foundOpponent = true;
			if((currentUnitInList->selfLearningTempVarBackPropagationPassError[currentPhase] < *currentLowestError))
			{
				*currentLowestError = currentUnitInList->selfLearningTempVarBackPropagationPassErrorCC;
				unitFound = currentUnitInList;
			}
		}

		if(currentUnitInList->isUnitGroup)
		{
			UnitListClass * tempUnit;
			double tempLowestError = SOME_LARGE_ERROR;
			tempUnit = searchUnitListFindOpponentWithLowestError(currentUnitInList->firstUnitInUnitGroup, &tempLowestError, foundOpponent);
			if(tempLowestError < *currentLowestError)
			{
				*currentLowestError = tempLowestError;
				unitFound = tempUnit;
			}
		}

		currentUnitInList = currentUnitInList->next;
	}

	return unitFound;
}

bool splitUnitGroup(UnitListClass * firstUnitInUnitList, string unitGroupName, int unitGroupColour, int currentRound)
{
	bool result = true;

	UnitListClass * currentUnitInList = firstUnitInUnitList;
	UnitListClass * unitGroupInList;

	bool foundUnitInList = false;

	while(currentUnitInList->next != NULL)
	{
		if((currentUnitInList->name == unitGroupName) && (currentUnitInList->team == unitGroupColour))
		{
			if(foundUnitInList)
			{
				cout << "Error: splitUnitGroup(); unit group found twice in unit list" << endl;
			}

			if(currentUnitInList->isUnitGroup == true)
			{
				foundUnitInList = true;

				currentUnitInList->roundRemoved = currentRound;
				currentUnitInList->unitDetails->status = false;
				unitGroupInList = currentUnitInList;
			}
			else
			{
				cout << "Error: splitUnitGroup(); unit in list is not a group" << endl;
				result = false;
			}
		}

		if(currentUnitInList->isUnitGroup == true)
		{
			if(!splitUnitGroup(currentUnitInList->firstUnitInUnitGroup, unitGroupName, unitGroupColour, currentRound))
			{
				result = false;
			}
		}

		if((foundUnitInList) && (currentUnitInList->next == NULL))
		{
			currentUnitInList->next = unitGroupInList->firstUnitInUnitGroup;
		}

		currentUnitInList = currentUnitInList->next;
	}

	return result;
}



